@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2017 2ndQuadrant Limited
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -23,13 +23,16 @@ import datetime
 import logging
 import os
 import shutil
+from contextlib import closing
 from glob import glob
 
 import dateutil.parser
 import dateutil.tz
 
 from barman import output, xlog
-from barman.backup_executor import PostgresBackupExecutor, RsyncBackupExecutor
+from barman.backup_executor import (PassiveBackupExecutor,
+                                    PostgresBackupExecutor,
+                                    RsyncBackupExecutor)
 from barman.compression import CompressionManager
 from barman.config import BackupOptions
 from barman.exceptions import (AbortedRetryHookScript,
@@ -37,9 +40,11 @@ from barman.exceptions import (AbortedRetryHookScript,
                                UnknownBackupIdException)
 from barman.hooks import HookScriptRunner, RetryHookScriptRunner
 from barman.infofile import BackupInfo, WalFileInfo
+from barman.lockfile import ServerBackupSyncLock
 from barman.recovery_executor import RecoveryExecutor
 from barman.remote_status import RemoteStatusMixin
-from barman.utils import fsync_dir, human_readable_timedelta, pretty_size
+from barman.utils import (fsync_dir, fsync_file, human_readable_timedelta,
+                          pretty_size)
 
 _logger = logging.getLogger(__name__)
 
@@ -47,7 +52,7 @@ _logger = logging.getLogger(__name__)
 class BackupManager(RemoteStatusMixin):
     """Manager of the backup archive for a server"""
 
-    DEFAULT_STATUS_FILTER = (BackupInfo.DONE,)
+    DEFAULT_STATUS_FILTER = BackupInfo.STATUS_COPY_DONE
 
     def __init__(self, server):
         """
@@ -60,7 +65,9 @@ class BackupManager(RemoteStatusMixin):
         self.compression_manager = CompressionManager(self.config, server.path)
         self.executor = None
         try:
-            if self.config.backup_method == "postgres":
+            if server.passive_node:
+                self.executor = PassiveBackupExecutor(self)
+            elif self.config.backup_method == "postgres":
                 self.executor = PostgresBackupExecutor(self)
             else:
                 self.executor = RsyncBackupExecutor(self)
@@ -168,8 +175,8 @@ class BackupManager(RemoteStatusMixin):
         if not isinstance(status_filter, tuple):
             status_filter = tuple(status_filter)
         backup = BackupInfo(self.server, backup_id=backup_id)
-        available_backups = self.get_available_backups(status_filter +
-                                                       (backup.status,))
+        available_backups = self.get_available_backups(
+            status_filter + (backup.status,))
         ids = sorted(available_backups.keys())
         try:
             current = ids.index(backup_id)
@@ -193,8 +200,8 @@ class BackupManager(RemoteStatusMixin):
         if not isinstance(status_filter, tuple):
             status_filter = tuple(status_filter)
         backup = BackupInfo(self.server, backup_id=backup_id)
-        available_backups = self.get_available_backups(status_filter +
-                                                       (backup.status,))
+        available_backups = self.get_available_backups(
+            status_filter + (backup.status,))
         ids = sorted(available_backups.keys())
         try:
             current = ids.index(backup_id)
@@ -243,8 +250,10 @@ class BackupManager(RemoteStatusMixin):
         Delete a backup
 
         :param backup: the backup to delete
+        :return bool: True if deleted, False if could not delete the backup
         """
-        available_backups = self.get_available_backups()
+        available_backups = self.get_available_backups(
+            status_filter=(BackupInfo.DONE,))
         minimum_redundancy = self.server.config.minimum_redundancy
         # Honour minimum required redundancy
         if backup.status == BackupInfo.DONE and \
@@ -255,11 +264,23 @@ class BackupManager(RemoteStatusMixin):
                            "current redundancy = %s)",
                            backup.backup_id,
                            self.config.name,
-                           len(available_backups),
-                           minimum_redundancy)
-            return
+                           minimum_redundancy,
+                           len(available_backups))
+            return False
         # Keep track of when the delete operation started.
         delete_start_time = datetime.datetime.now()
+
+        # Run the pre_delete_script if present.
+        script = HookScriptRunner(self, 'delete_script', 'pre')
+        script.env_from_backup_info(backup)
+        script.run()
+
+        # Run the pre_delete_retry_script if present.
+        retry_script = RetryHookScriptRunner(
+            self, 'delete_retry_script', 'pre')
+        retry_script.env_from_backup_info(backup)
+        retry_script.run()
+
         output.info("Deleting backup %s for server %s",
                     backup.backup_id, self.config.name)
         previous_backup = self.get_previous_backup(backup.backup_id)
@@ -270,7 +291,7 @@ class BackupManager(RemoteStatusMixin):
         except OSError as e:
             output.error("Failure deleting backup %s for server %s.\n%s",
                          backup.backup_id, self.config.name, e)
-            return
+            return False
         # Check if we are deleting the first available backup
         if not previous_backup:
             # In the case of exclusive backup (default), removes any WAL
@@ -314,7 +335,7 @@ class BackupManager(RemoteStatusMixin):
                          "Please manually remove the '%s' directory",
                          backup.backup_id, self.config.name, e,
                          backup.get_basebackup_directory())
-            return
+            return False
         self.backup_cache_remove(backup)
         # Save the time of the complete removal of the backup
         delete_end_time = datetime.datetime.now()
@@ -324,9 +345,40 @@ class BackupManager(RemoteStatusMixin):
                     human_readable_timedelta(
                         delete_end_time - delete_start_time))
 
+        # Remove the sync lockfile if exists
+        sync_lock = ServerBackupSyncLock(self.config.barman_lock_directory,
+                                         self.config.name, backup.backup_id)
+        if os.path.exists(sync_lock.filename):
+            _logger.debug("Deleting backup sync lockfile: %s" %
+                          sync_lock.filename)
+
+            os.unlink(sync_lock.filename)
+
+        # Run the post_delete_retry_script if present.
+        try:
+            retry_script = RetryHookScriptRunner(
+                self, 'delete_retry_script', 'post')
+            retry_script.env_from_backup_info(backup)
+            retry_script.run()
+        except AbortedRetryHookScript as e:
+            # Ignore the ABORT_STOP as it is a post-hook operation
+            _logger.warning("Ignoring stop request after receiving "
+                            "abort (exit code %d) from post-delete "
+                            "retry hook script: %s",
+                            e.hook.exit_status, e.hook.script)
+
+        # Run the post_delete_script if present.
+        script = HookScriptRunner(self, 'delete_script', 'post')
+        script.env_from_backup_info(backup)
+        script.run()
+
+        return True
+
     def backup(self):
         """
         Performs a backup for the server
+
+        :return BackupInfo: the generated BackupInfo
         """
         _logger.debug("initialising backup information")
         self.executor.init()
@@ -361,10 +413,10 @@ class BackupManager(RemoteStatusMixin):
             # Compute backup size and fsync it on disk
             self.backup_fsync_and_set_sizes(backup_info)
 
-            # Mark the backup as DONE
-            backup_info.set_attribute("status", "DONE")
+            # Mark the backup as WAITING_FOR_WALS
+            backup_info.set_attribute("status", BackupInfo.WAITING_FOR_WALS)
         # Use BaseException instead of Exception to catch events like
-        # KeyboardInterrupt (e.g.: CRTL-C)
+        # KeyboardInterrupt (e.g.: CTRL-C)
         except BaseException as e:
             msg_lines = str(e).strip().splitlines()
             if backup_info:
@@ -389,11 +441,13 @@ class BackupManager(RemoteStatusMixin):
                         backup_info.end_xlog,
                         backup_info.end_wal,
                         backup_info.end_offset)
-            output.info("Backup completed (start time: %s, elapsed time: %s)",
-                        self.executor.copy_start_time,
-                        human_readable_timedelta(
-                            self.executor.copy_end_time
-                            - self.executor.copy_start_time))
+
+            executor = self.executor
+            output.info(
+                "Backup completed (start time: %s, elapsed time: %s)",
+                self.executor.copy_start_time,
+                human_readable_timedelta(
+                    executor.copy_end_time - executor.copy_start_time))
             # Create a restore point after a backup
             target_name = 'barman_%s' % backup_info.backup_id
             self.server.postgres.create_restore_point(target_name)
@@ -424,10 +478,10 @@ class BackupManager(RemoteStatusMixin):
                 script.run()
 
         output.result('backup', backup_info)
+        return backup_info
 
-    def recover(self, backup_info, dest, tablespaces=None, target_tli=None,
-                target_time=None, target_xid=None, target_name=None,
-                target_immediate=False, exclusive=False, remote_command=None):
+    def recover(self, backup_info, dest, tablespaces=None, remote_command=None,
+                **kwargs):
         """
         Performs a recovery of a backup
 
@@ -435,28 +489,67 @@ class BackupManager(RemoteStatusMixin):
         :param str dest: the destination directory
         :param dict[str,str]|None tablespaces: a tablespace name -> location
             map (for relocation)
-        :param str|None target_tli: the target timeline
-        :param str|None target_time: the target time
-        :param str|None target_xid: the target xid
-        :param str|None target_name: the target name created previously with
-            pg_create_restore_point() function call
-        :param bool|None target_immediate: end recovery as soon as consistency
-            is reached
-        :param bool exclusive: whether the recovery is exclusive or not
         :param str|None remote_command: default None. The remote command
             to recover the base backup, in case of remote backup.
+        :kwparam str|None target_tli: the target timeline
+        :kwparam str|None target_time: the target time
+        :kwparam str|None target_xid: the target xid
+        :kwparam str|None target_name: the target name created previously with
+            pg_create_restore_point() function call
+        :kwparam bool|None target_immediate: end recovery as soon as
+            consistency is reached
+        :kwparam bool exclusive: whether the recovery is exclusive or not
+        :kwparam str|None target_action: default None. The recovery target
+            action
+        :kwparam bool|None standby_mode: the standby mode if needed
         """
 
         # Archive every WAL files in the incoming directory of the server
         self.server.archive_wal(verbose=False)
         # Delegate the recovery operation to a RecoveryExecutor object
         executor = RecoveryExecutor(self)
-        recovery_info = executor.recover(backup_info,
-                                         dest, tablespaces,
-                                         target_tli, target_time,
-                                         target_xid, target_name,
-                                         target_immediate, exclusive,
-                                         remote_command)
+
+        # Run the pre_recovery_script if present.
+        script = HookScriptRunner(self, 'recovery_script', 'pre')
+        script.env_from_recover(backup_info, dest, tablespaces, remote_command,
+                                **kwargs)
+        script.run()
+
+        # Run the pre_recovery_retry_script if present.
+        retry_script = RetryHookScriptRunner(
+            self, 'recovery_retry_script', 'pre')
+        script.env_from_recover(backup_info, dest, tablespaces, remote_command,
+                                **kwargs)
+        retry_script.run()
+
+        # Execute the recovery.
+        # We use a closing context to automatically remove
+        # any resource eventually allocated during recovery.
+        with closing(executor):
+            recovery_info = executor.recover(
+                backup_info, dest,
+                tablespaces=tablespaces, remote_command=remote_command,
+                **kwargs)
+
+        # Run the post_recovery_retry_script if present.
+        try:
+            retry_script = RetryHookScriptRunner(
+                self, 'recovery_retry_script', 'post')
+            script.env_from_recover(
+                backup_info, dest, tablespaces, remote_command, **kwargs)
+            retry_script.run()
+        except AbortedRetryHookScript as e:
+            # Ignore the ABORT_STOP as it is a post-hook operation
+            _logger.warning("Ignoring stop request after receiving "
+                            "abort (exit code %d) from post-recovery "
+                            "retry hook script: %s",
+                            e.hook.exit_status, e.hook.script)
+
+        # Run the post-recovery-script if present.
+        script = HookScriptRunner(self, 'recovery_script', 'post')
+        script.env_from_recover(backup_info, dest, tablespaces, remote_command,
+                                **kwargs)
+        script.run()
 
         # Output recovery results
         output.result('recovery', recovery_info['results'])
@@ -477,8 +570,9 @@ class BackupManager(RemoteStatusMixin):
         """
         Retention policy management
         """
-        if (self.server.enforce_retention_policies and
-                self.config.retention_policy_mode == 'auto'):
+        enforce_retention_policies = self.server.enforce_retention_policies
+        retention_policy_mode = self.config.retention_policy_mode
+        if (enforce_retention_policies and retention_policy_mode == 'auto'):
             available_backups = self.get_available_backups(
                 BackupInfo.STATUS_ALL)
             retention_status = self.config.retention_policy.report()
@@ -530,6 +624,18 @@ class BackupManager(RemoteStatusMixin):
         :param barman.infofile.WalFileInfo wal_info: the WAL to delete
         """
 
+        # Run the pre_wal_delete_script if present.
+        script = HookScriptRunner(self, 'wal_delete', 'pre')
+        script.env_from_wal_info(wal_info)
+        script.run()
+
+        # Run the pre_wal_delete_retry_script if present.
+        retry_script = RetryHookScriptRunner(
+            self, 'wal_delete_retry_script', 'pre')
+        script.env_from_wal_info(wal_info)
+        retry_script.run()
+
+        error = None
         try:
             os.unlink(wal_info.fullpath(self.server))
             try:
@@ -540,9 +646,27 @@ class BackupManager(RemoteStatusMixin):
                 # this means that hashdir is not empty.
                 pass
         except OSError as e:
-            output.warning('Ignoring deletion of WAL file %s '
-                           'for server %s: %s',
-                           wal_info.name, self.config.name, e)
+            error = ('Ignoring deletion of WAL file %s for server %s: %s' %
+                     (wal_info.name, self.config.name, e))
+            output.warning(error)
+
+        # Run the post_wal_delete_retry_script if present.
+        try:
+            retry_script = RetryHookScriptRunner(
+                self, 'wal_delete_retry_script', 'post')
+            script.env_from_wal_info(wal_info, None, error)
+            retry_script.run()
+        except AbortedRetryHookScript as e:
+            # Ignore the ABORT_STOP as it is a post-hook operation
+            _logger.warning("Ignoring stop request after receiving "
+                            "abort (exit code %d) from post-wal-delete "
+                            "retry hook script: %s",
+                            e.hook.exit_status, e.hook.script)
+
+        # Run the post_wal_delete_script if present.
+        script = HookScriptRunner(self, 'wal_delete', 'post')
+        script.env_from_wal_info(wal_info, None, error)
+        script.run()
 
     def check(self, check_strategy):
         """
@@ -558,7 +682,7 @@ class BackupManager(RemoteStatusMixin):
         else:
             status = True
             try:
-                self.compression_manager.get_compressor()
+                self.compression_manager.get_default_compressor()
             except CompressionIncompatibility as field:
                 check_strategy.result(self.config.name,
                                       '%s setting' % field, False)
@@ -576,7 +700,8 @@ class BackupManager(RemoteStatusMixin):
         )
         check_strategy.init_check('minimum redundancy requirements')
         # Minimum redundancy checks
-        no_backups = len(self.get_available_backups())
+        no_backups = len(self.get_available_backups(
+            status_filter=(BackupInfo.DONE,)))
         # Check minimum_redundancy_requirements parameter
         if no_backups < int(self.config.minimum_redundancy):
             status = False
@@ -598,7 +723,8 @@ class BackupManager(RemoteStatusMixin):
         This function show the server status
         """
         # get number of backups
-        no_backups = len(self.get_available_backups())
+        no_backups = len(self.get_available_backups(
+            status_filter=(BackupInfo.DONE,)))
         output.result('status', self.config.name,
                       "backups_number",
                       "No. of available backups", no_backups)
@@ -653,7 +779,7 @@ class BackupManager(RemoteStatusMixin):
 
         output.info("Rebuilding xlogdb for server %s", self.config.name)
         root = self.config.wals_directory
-        default_compression = self.config.compression
+        comp_manager = self.compression_manager
         wal_count = label_count = history_count = 0
         # lock the xlogdb as we are about replacing it completely
         with self.server.xlogdb('w') as fxlogdb:
@@ -691,17 +817,15 @@ class BackupManager(RemoteStatusMixin):
                                         'rebuilding the wal database: %s',
                                         fullname)
                                     continue
-                                wal_info = WalFileInfo.from_file(
-                                    fullname,
-                                    default_compression=default_compression)
+                                wal_info = comp_manager.get_wal_file_info(
+                                    fullname)
                                 fxlogdb_new.write(wal_info.to_xlogdb_line())
                     else:
                         # only history files are here
                         if xlog.is_history_file(fullname):
                             history_count += 1
-                            wal_info = WalFileInfo.from_file(
-                                fullname,
-                                default_compression=default_compression)
+                            wal_info = comp_manager.get_wal_file_info(
+                                fullname)
                             fxlogdb_new.write(wal_info.to_xlogdb_line())
                         else:
                             _logger.warning(
@@ -726,10 +850,11 @@ class BackupManager(RemoteStatusMixin):
         from os.path import isdir, join
 
         root = self.config.wals_directory
+        comp_manager = self.compression_manager
 
         # If the WAL archive directory doesn't exists the archive is empty
         if not isdir(root):
-            return None
+            return dict()
 
         # Traverse all the directory in the archive in reverse order,
         # returning the first WAL file found
@@ -756,11 +881,12 @@ class BackupManager(RemoteStatusMixin):
                     fullname = join(hash_dir, wal_name)
                     # Return the first file that has the correct name
                     if not isdir(fullname) and xlog.is_wal_file(fullname):
-                        timelines[timeline] = WalFileInfo.from_file(fullname)
+                        timelines[timeline] = comp_manager.get_wal_file_info(
+                            fullname)
                         break
 
-        # Return the timeline map or None if it is empty
-        return timelines or None
+        # Return the timeline map
+        return timelines
 
     def remove_wal_before_backup(self, backup_info, timelines_to_protect=None):
         """
@@ -873,14 +999,11 @@ class BackupManager(RemoteStatusMixin):
             # execute fsync() on all the contained files
             for filename in file_names:
                 file_path = os.path.join(dir_path, filename)
-                file_fd = os.open(file_path, os.O_RDONLY)
-                file_stat = os.fstat(file_fd)
+                file_stat = fsync_file(file_path)
                 backup_size += file_stat.st_size
                 # Excludes hard links from real backup size
                 if file_stat.st_nlink == 1:
                     deduplicated_size += file_stat.st_size
-                os.fsync(file_fd)
-                os.close(file_fd)
         # Save size into BackupInfo object
         backup_info.set_attribute('size', backup_size)
         backup_info.set_attribute('deduplicated_size', deduplicated_size)
@@ -901,3 +1024,84 @@ class BackupManager(RemoteStatusMixin):
         else:
             output.info("Backup size: %s" %
                         pretty_size(backup_info.size))
+
+    def check_backup(self, backup_info):
+        """
+        Make sure that all the required WAL files to check
+        the consistency of a physical backup (that is, from the
+        beginning to the end of the full backup) are correctly
+        archived. This command is automatically invoked by the
+        cron command and at the end of every backup operation.
+
+        :param backup_info: the target backup
+        """
+
+        # Gather the list of the latest archived wals
+        timelines = self.get_latest_archived_wals_info()
+
+        # Get the basic info for the backup
+        begin_wal = backup_info.begin_wal
+        end_wal = backup_info.end_wal
+        timeline = begin_wal[:8]
+
+        # Case 1: Barman still doesn't know about the timeline the backup
+        # started with. We still haven't archived any WAL corresponding
+        # to the backup, so we can't proceed with checking the existence
+        # of the required WAL files
+        if not timelines or timeline not in timelines:
+            backup_info.status = BackupInfo.WAITING_FOR_WALS
+            backup_info.save()
+            return
+
+        # Find the most recent archived WAL for this server in the timeline
+        # where the backup was taken
+        last_archived_wal = timelines[timeline].name
+
+        # Case 2: the most recent WAL file archived is older than the
+        # start of the backup. We must wait for the archiver to receive
+        # and/or process the WAL files.
+        if last_archived_wal < begin_wal:
+            backup_info.status = BackupInfo.WAITING_FOR_WALS
+            backup_info.save()
+            return
+
+        # Check the intersection between the required WALs and the archived
+        # ones. They should all exist
+        bound_end = min(last_archived_wal, end_wal)
+        segments = xlog.generate_segment_names(
+            begin_wal,
+            bound_end,
+            xlog_segment_size=backup_info.xlog_segment_size)
+        missing_wal = None
+        for wal in segments:
+            wal_full_path = self.server.get_wal_full_path(wal)
+            if not os.path.exists(wal_full_path):
+                missing_wal = wal
+                break
+
+        if missing_wal:
+            # Case 3: the most recent WAL file archived is more recent than
+            # the one corresponding to the start of a backup. If WAL
+            # file is missing, then we can't recover from the backup so we
+            # must mark the backup as FAILED.
+            # TODO: Verify if the error field is the right place
+            # to store the error message
+            backup_info.error = (
+                "At least one WAL file is missing. "
+                "The first missing WAL file is %s" % missing_wal)
+            backup_info.status = BackupInfo.FAILED
+            backup_info.save()
+            return
+
+        if end_wal <= last_archived_wal:
+            # Case 4: if the most recent WAL file archived is more recent or
+            # equal than the one corresponding to the end of the backup and
+            # every WAL that will be required by the recovery is available,
+            # we can mark the backup as DONE.
+            backup_info.status = BackupInfo.DONE
+        else:
+            # Case 5: if the most recent WAL file archived is older than
+            # the one corresponding to the end of the backup but
+            # all the WAL files until that point are present.
+            backup_info.status = BackupInfo.WAITING_FOR_WALS
+        backup_info.save()
