@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2017 2ndQuadrant Limited
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -310,12 +310,12 @@ class PostgresBackupExecutor(BackupExecutor):
             backup_info.save()
 
             if backup_info.begin_wal is not None:
-                output.info("Backup start at xlog location: %s (%s, %08X)",
+                output.info("Backup start at LSN: %s (%s, %08X)",
                             backup_info.begin_xlog,
                             backup_info.begin_wal,
                             backup_info.begin_offset)
             else:
-                output.info("Backup start at xlog location: %s",
+                output.info("Backup start at LSN: %s",
                             backup_info.begin_xlog)
 
             # Start the copy
@@ -661,10 +661,12 @@ class SshBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
                 'for server %s' % backup_manager.config.name)
 
         # Apply the default backup strategy
-        if (BackupOptions.CONCURRENT_BACKUP not in
-                self.config.backup_options and
-                BackupOptions.EXCLUSIVE_BACKUP not in
-                self.config.backup_options):
+        backup_options = self.config.backup_options
+        concurrent_backup = (
+            BackupOptions.CONCURRENT_BACKUP in backup_options)
+        exclusive_backup = (
+            BackupOptions.EXCLUSIVE_BACKUP in backup_options)
+        if not concurrent_backup and not exclusive_backup:
             self.config.backup_options.add(BackupOptions.EXCLUSIVE_BACKUP)
             output.debug("The default backup strategy for "
                          "any ssh based backup_method is: "
@@ -723,12 +725,12 @@ class SshBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
             backup_info.save()
 
             if backup_info.begin_wal is not None:
-                output.info("Backup start at xlog location: %s (%s, %08X)",
+                output.info("Backup start at LSN: %s (%s, %08X)",
                             backup_info.begin_xlog,
                             backup_info.begin_wal,
                             backup_info.begin_offset)
             else:
-                output.info("Backup start at xlog location: %s",
+                output.info("Backup start at LSN: %s",
                             backup_info.begin_xlog)
 
             # If this is the first backup, purge eventually unused WAL files
@@ -744,7 +746,7 @@ class SshBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
             # the begin_wal value is surely known. Doing it twice is safe
             # because this function is useful only during the first backup.
             self._purge_unused_wal_files(backup_info)
-        except:
+        except BaseException:
             # we do not need to do anything here besides re-raising the
             # exception. It will be handled in the external try block.
             output.error("The backup has failed %s", self.current_action)
@@ -778,7 +780,7 @@ class SshBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
                                     path=self.server.path)
             minimal_ssh_output = ''.join(cmd.get_last_output())
         except FsOperationFailed as e:
-                hint = str(e).strip()
+            hint = str(e).strip()
 
         # Output the result
         check_strategy.result(self.config.name, cmd is not None, hint=hint)
@@ -793,9 +795,9 @@ class SshBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
                      "the remote command output")
 
         # If SSH works but PostgreSQL is not responding
-        if (cmd is not None and
-                self.server.get_remote_status()['server_txt_version']
-                is None):
+        server_txt_version = self.server.get_remote_status().get(
+            'server_txt_version')
+        if cmd is not None and server_txt_version is None:
             # Check for 'backup_label' presence
             last_backup = self.server.get_backup(
                 self.server.get_last_backup_id(BackupInfo.STATUS_NOT_EMPTY)
@@ -857,6 +859,9 @@ class SshBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
                     cmd = UnixRemoteCommand(self.ssh_command,
                                             self.ssh_options,
                                             path=self.server.path)
+                    # Here the name of the PostgreSQL WALs directory is
+                    # hardcoded, but that doesn't represent a problem as
+                    # this code runs only for PostgreSQL < 9.4
                     archive_dir = os.path.join(
                         self.server.postgres.get_setting('data_directory'),
                         'pg_xlog', 'archive_status')
@@ -880,6 +885,95 @@ class SshBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
         output.info(message)
 
 
+class PassiveBackupExecutor(BackupExecutor):
+    """
+    Dummy backup executors for Passive servers.
+
+    Raises a SshCommandException if 'primary_ssh_command' is not set.
+    """
+
+    def __init__(self, backup_manager):
+        """
+        Constructor of Dummy backup executors for Passive servers.
+
+        :param barman.backup.BackupManager backup_manager: the BackupManager
+            assigned to the executor
+        """
+        super(PassiveBackupExecutor, self).__init__(backup_manager)
+
+        # Retrieve the ssh command and the options necessary for the
+        # remote ssh access.
+        self.ssh_command, self.ssh_options = _parse_ssh_command(
+            backup_manager.config.primary_ssh_command)
+
+        # Requires ssh_command to be set
+        if not self.ssh_command:
+            raise SshCommandException(
+                'Invalid primary_ssh_command in barman configuration '
+                'for server %s' % backup_manager.config.name)
+
+    def backup(self, backup_info):
+        """
+        This method should never be called, because this is a passive server
+
+        :param barman.infofile.BackupInfo backup_info: backup information
+        """
+        # The 'backup' command is not available on a passive node.
+        # If we get here, there is a programming error
+        assert False
+
+    def check(self, check_strategy):
+        """
+        Perform additional checks for PassiveBackupExecutor, including
+        Ssh connection to the primary (executing a 'true' command on the
+        remote server).
+
+        :param CheckStrategy check_strategy: the strategy for the management
+             of the results of the various checks
+        """
+        check_strategy.init_check('ssh')
+        hint = 'Barman primary node'
+        cmd = None
+        minimal_ssh_output = None
+        try:
+            cmd = UnixRemoteCommand(self.ssh_command,
+                                    self.ssh_options,
+                                    path=self.server.path)
+            minimal_ssh_output = ''.join(cmd.get_last_output())
+        except FsOperationFailed as e:
+            hint = str(e).strip()
+
+        # Output the result
+        check_strategy.result(self.config.name, cmd is not None, hint=hint)
+
+        # Check if the communication channel is "clean"
+        if minimal_ssh_output:
+            check_strategy.init_check('ssh output clean')
+            check_strategy.result(
+                self.config.name,
+                False,
+                hint="the configured ssh_command must not add anything to "
+                     "the remote command output")
+
+    def status(self):
+        """
+        Set additional status info for PassiveBackupExecutor.
+        """
+        # On passive nodes show the primary_ssh_command
+        output.result('status', self.config.name,
+                      "primary_ssh_command",
+                      "SSH command to primary server",
+                      self.config.primary_ssh_command)
+
+    @property
+    def mode(self):
+        """
+        Property that defines the mode used for the backup.
+        :return str: a string describing the mode used for the backup
+        """
+        return 'passive'
+
+
 class RsyncBackupExecutor(SshBackupExecutor):
     """
     Concrete class for backup via Rsync+Ssh.
@@ -1132,10 +1226,10 @@ class BackupStrategy(with_metaclass(ABCMeta, object)):
     """
 
     #: Regex for START WAL LOCATION info
-    START_TIME_RE = re.compile('^START TIME: (.*)', re.MULTILINE)
+    START_TIME_RE = re.compile(r'^START TIME: (.*)', re.MULTILINE)
 
     #: Regex for START TIME info
-    WAL_RE = re.compile('^START WAL LOCATION: (.*) \(file (.*)\)',
+    WAL_RE = re.compile(r'^START WAL LOCATION: (.*) \(file (.*)\)',
                         re.MULTILINE)
 
     def __init__(self, executor, mode=None):
@@ -1248,8 +1342,9 @@ class BackupStrategy(with_metaclass(ABCMeta, object)):
                 backup_info.xlog_segment_size))
 
         # If file_name and file_offset are available, use them
-        if (start_info.get('file_name') is not None and
-                start_info.get('file_offset') is not None):
+        file_name = start_info.get('file_name')
+        file_offset = start_info.get('file_offset')
+        if (file_name is not None and file_offset is not None):
             backup_info.set_attribute('begin_wal',
                                       start_info['file_name'])
             backup_info.set_attribute('begin_offset',
@@ -1273,8 +1368,9 @@ class BackupStrategy(with_metaclass(ABCMeta, object)):
 
         # If file_name or file_offset are missing build them using the stop
         # location and the timeline.
-        if (stop_info.get('file_name') is None or
-                stop_info.get('file_offset') is None):
+        file_name = stop_info.get('file_name')
+        file_offset = stop_info.get('file_offset')
+        if file_name is None or file_offset is None:
             # Take a copy of stop_info because we are going to update it
             stop_info = stop_info.copy()
             # Get the timeline from the stop_info if available, otherwise
@@ -1382,11 +1478,11 @@ class PostgresBackupStrategy(BackupStrategy):
             self._backup_info_from_stop_location(
                 backup_info, current_xlog_info)
 
-        # Ask PostgreSQL to switch to another XLOG file. This is needed
+        # Ask PostgreSQL to switch to another WAL file. This is needed
         # to archive the transaction log file containing the backup
         # end position, which is required to recover from the backup.
         try:
-            postgres.switch_xlog()
+            postgres.switch_wal()
         except PostgresIsInRecovery:
             # Skip switching XLOG if a standby server
             pass
@@ -1556,12 +1652,15 @@ class ConcurrentBackupStrategy(BackupStrategy):
         :param barman.infofile.BackupInfo backup_info: backup information
         """
         pg_version = self.executor.server.postgres.server_version
+        self.current_action = "issuing stop backup command"
         if pg_version >= 90600:
             # On 9.6+ execute native concurrent stop backup
+            self.current_action += " (native concurrent)"
             _logger.debug("Stop of native concurrent backup")
             self._concurrent_stop_backup(backup_info)
         else:
             # On older Postgres use pgespresso
+            self.current_action += " (pgespresso)"
             _logger.debug("Stop of concurrent backup with pgespresso")
             self._pgespresso_stop_backup(backup_info)
 
@@ -1569,12 +1668,12 @@ class ConcurrentBackupStrategy(BackupStrategy):
         self.current_action = "writing backup label"
         self._write_backup_label(backup_info)
 
-        # Ask PostgreSQL to switch to another XLOG file. This is needed
+        # Ask PostgreSQL to switch to another WAL file. This is needed
         # to archive the transaction log file containing the backup
         # end position, which is required to recover from the backup.
         postgres = self.executor.server.postgres
         try:
-            postgres.switch_xlog()
+            postgres.switch_wal()
         except PostgresIsInRecovery:
             # Skip switching XLOG if a standby server
             pass
