@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2017 2ndQuadrant Limited
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -19,6 +19,7 @@
 This module implements the interface with the command line and the logger.
 """
 
+import json
 import logging
 import os
 import sys
@@ -32,10 +33,11 @@ import barman.config
 import barman.diagnose
 from barman import output
 from barman.config import RecoveryOptions
-from barman.exceptions import BadXlogSegmentName
+from barman.exceptions import BadXlogSegmentName, RecoveryException, SyncError
 from barman.infofile import BackupInfo
 from barman.server import Server
-from barman.utils import configure_logging, drop_privileges, parse_log_level
+from barman.utils import (BarmanEncoder, configure_logging, drop_privileges,
+                          parse_log_level)
 
 _logger = logging.getLogger(__name__)
 
@@ -78,6 +80,22 @@ def check_positive(value):
     return int_value
 
 
+def check_target_action(value):
+    """
+    Check the target action option
+
+    :param value: str containing the value to check
+    """
+    if value is None:
+        return None
+
+    if value in ('pause', 'shutdown', 'promote'):
+        return value
+
+    raise ArgumentTypeError("'%s' is not a valid recovery target action" %
+                            value)
+
+
 @named('list-server')
 @arg('--minimal', help='machine readable output')
 def list_server(minimal=False):
@@ -102,6 +120,9 @@ def list_server(minimal=False):
         # If server has configuration errors
         elif server.config.disabled:
             description += " (WARNING: disabled)"
+        # If server is a passive node
+        if server.passive_node:
+            description += ' (Passive)'
         output.result('list_server', name, description)
     output.close_and_exit()
 
@@ -193,7 +214,7 @@ def backup(args):
     """
     Perform a full backup for the given server (supports 'all')
     """
-    servers = get_server_list(args, skip_inactive=True)
+    servers = get_server_list(args, skip_inactive=True, skip_passive=True)
     for name in sorted(servers):
         server = servers[name]
 
@@ -379,6 +400,21 @@ def rebuild_xlogdb(args):
      dest='network_compression',
      action='store_false',
      default=SUPPRESS)
+@arg('--target-action',
+     help='Specifies what action the server should take once the '
+          'recovery target is reached. This option is not allowed for '
+          'PostgreSQL < 9.1. If PostgreSQL is between 9.1 and 9.4 included '
+          'the only allowed value is "pause". If PostgreSQL is 9.5 or newer '
+          'the possible values are "shutdown", "pause", "promote".',
+     dest='target_action',
+     type=check_target_action,
+     default=SUPPRESS)
+@arg('--standby-mode',
+     dest="standby_mode",
+     action='store_true',
+     default=SUPPRESS,
+     help='Enable standby mode when starting '
+          'the recovered PostgreSQL instance')
 @expects_obj
 def recover(args):
     """
@@ -388,7 +424,7 @@ def recover(args):
 
     # Retrieves the backup
     backup_id = parse_backup_id(server, args)
-    if backup_id.status != BackupInfo.DONE:
+    if backup_id.status not in BackupInfo.STATUS_COPY_DONE:
         output.error(
             "Cannot recover from backup '%s' of server '%s': "
             "backup status is not DONE",
@@ -469,17 +505,23 @@ def recover(args):
                 "you have to use the --remote-ssh-command option")
             output.close_and_exit()
         server.config.network_compression = args.network_compression
+
     with closing(server):
-        server.recover(backup_id,
-                       args.destination_directory,
-                       tablespaces=tablespaces,
-                       target_tli=args.target_tli,
-                       target_time=args.target_time,
-                       target_xid=args.target_xid,
-                       target_name=args.target_name,
-                       target_immediate=args.target_immediate,
-                       exclusive=args.exclusive,
-                       remote_command=args.remote_ssh_command)
+        try:
+            server.recover(backup_id,
+                           args.destination_directory,
+                           tablespaces=tablespaces,
+                           target_tli=args.target_tli,
+                           target_time=args.target_time,
+                           target_xid=args.target_xid,
+                           target_name=args.target_name,
+                           target_immediate=args.target_immediate,
+                           exclusive=args.exclusive,
+                           remote_command=args.remote_ssh_command,
+                           target_action=getattr(args, 'target_action', None),
+                           standby_mode=getattr(args, 'standby_mode', None))
+        except RecoveryException as exc:
+            output.error(str(exc))
 
     output.close_and_exit()
 
@@ -611,6 +653,84 @@ def diagnose():
     output.close_and_exit()
 
 
+@named('sync-info')
+@arg('--primary', help='execute the sync-info on the primary node (if set)',
+     action='store_true', default=SUPPRESS)
+@arg("server_name",
+     completer=server_completer,
+     help='specifies the server name for the command')
+@arg("last_wal",
+     help='specifies the name of the latest WAL read',
+     nargs='?')
+@arg("last_position",
+     nargs='?',
+     type=check_positive,
+     help='the last position read from xlog database (in bytes)')
+@expects_obj
+def sync_info(args):
+    """
+    Output the internal synchronisation status.
+    Used to sync_backup with a passive node
+    """
+    server = get_server(args)
+    try:
+        # if called with --primary option
+        if getattr(args, 'primary', False):
+            primary_info = server.primary_node_info(args.last_wal,
+                                                    args.last_position)
+            output.info(json.dumps(primary_info, sys.stdout,
+                                   cls=BarmanEncoder, indent=4),
+                        log=False)
+        else:
+            server.sync_status(args.last_wal, args.last_position)
+    except SyncError as e:
+        # Catch SyncError exceptions and output only the error message,
+        # preventing from logging the stack trace
+        output.error(e)
+
+    output.close_and_exit()
+
+
+@named('sync-backup')
+@arg("server_name",
+     completer=server_completer,
+     help='specifies the server name for the command')
+@arg("backup_id",
+     help='specifies the backup ID to be copied on the passive node')
+@expects_obj
+def sync_backup(args):
+    """
+    Command that synchronises a backup from a master to a passive node
+    """
+    server = get_server(args)
+    try:
+        server.sync_backup(args.backup_id)
+    except SyncError as e:
+        # Catch SyncError exceptions and output only the error message,
+        # preventing from logging the stack trace
+        output.error(e)
+    output.close_and_exit()
+
+
+@named('sync-wals')
+@arg("server_name",
+     completer=server_completer,
+     help='specifies the server name for the command')
+@expects_obj
+def sync_wals(args):
+    """
+    Command that synchronises WAL files from a master to a passive node
+    """
+    server = get_server(args)
+    try:
+        server.sync_wals()
+    except SyncError as e:
+        # Catch SyncError exceptions and output only the error message,
+        # preventing from logging the stack trace
+        output.error(e)
+    output.close_and_exit()
+
+
 @named('show-backup')
 @arg('server_name',
      completer=server_completer,
@@ -684,7 +804,9 @@ def delete(args):
     # Retrieves the backup
     backup_id = parse_backup_id(server, args)
     with closing(server):
-        server.delete_backup(backup_id)
+        if not server.delete_backup(backup_id):
+            output.error("Cannot delete backup (%s %s)"
+                         % (server.config.name, backup_id))
     output.close_and_exit()
 
 
@@ -736,6 +858,28 @@ def get_wal(args):
     output.close_and_exit()
 
 
+@named('put-wal')
+@arg('server_name',
+     completer=server_completer,
+     help='specifies the server name for the command')
+@expects_obj
+def put_wal(args):
+    """
+    Receive a WAL file from SERVER_NAME and securely store it in the incoming
+    directory. The file will be read from standard input in tar format.
+    """
+    server = get_server(args, inactive_is_error=True)
+    try:
+        # Python 3.x
+        stream = sys.stdin.buffer
+    except AttributeError:
+        # Python 2.x
+        stream = sys.stdin
+    with closing(server):
+        server.put_wal(stream)
+    output.close_and_exit()
+
+
 @named('archive-wal')
 @arg('server_name',
      completer=server_completer,
@@ -793,6 +937,32 @@ def receive_wal(args):
     output.close_and_exit()
 
 
+@named('check-backup')
+@arg('server_name',
+     completer=server_completer,
+     help='specifies the server name for the command')
+@arg('backup_id',
+     completer=backup_completer,
+     help='specifies the backup ID')
+@expects_obj
+def check_backup(args):
+    """
+    Make sure that all the required WAL files to check
+    the consistency of a physical backup (that is, from the
+    beginning to the end of the full backup) are correctly
+    archived. This command is automatically invoked by the
+    cron command and at the end of every backup operation.
+    """
+    server = get_server(args)
+
+    # Retrieves the backup
+    backup_info = parse_backup_id(server, args)
+
+    with closing(server):
+        server.check_backup(backup_info)
+    output.close_and_exit()
+
+
 def pretty_args(args):
     """
     Prettify the given argh namespace to be human readable
@@ -844,12 +1014,20 @@ def global_config(args):
     if log_level is None:
         _logger.warn('unknown log_level in config file: %s', config.log_level)
 
-    # configure output
+    # Configure output
     if args.format != output.DEFAULT_WRITER or args.quiet or args.debug:
         output.set_output_writer(args.format,
                                  quiet=args.quiet,
                                  debug=args.debug)
 
+    # Configure color output
+    if args.color == 'auto':
+        # Enable colored output if both stdout and stderr are TTYs
+        output.ansi_colors_enabled = (
+            sys.stdout.isatty() and sys.stderr.isatty())
+    else:
+        output.ansi_colors_enabled = args.color == 'always'
+
     # Load additional configuration files
     config.load_configuration_files_directory()
     # We must validate the configuration here in order to have
@@ -861,6 +1039,7 @@ def global_config(args):
 
 
 def get_server(args, skip_inactive=True, skip_disabled=False,
+               skip_passive=False,
                inactive_is_error=False,
                on_error_stop=True, suppress_error=False):
     """
@@ -876,6 +1055,7 @@ def get_server(args, skip_inactive=True, skip_disabled=False,
         WARNING: the function modifies the content of this parameter
     :param bool skip_inactive: do nothing if the server is inactive
     :param bool skip_disabled: do nothing if the server is disabled
+    :param bool skip_passive: do nothing if the server is passive
     :param bool inactive_is_error: treat inactive server as error
     :param bool on_error_stop: stop if an error is found
     :param bool suppress_error: suppress display of errors (e.g. diagnose)
@@ -902,6 +1082,7 @@ def get_server(args, skip_inactive=True, skip_disabled=False,
 
     # Retrieve the requested server
     servers = get_server_list(args, skip_inactive, skip_disabled,
+                              skip_passive,
                               on_error_stop, suppress_error)
 
     # The requested server has been excluded from get_server_list result
@@ -930,6 +1111,7 @@ def get_server(args, skip_inactive=True, skip_disabled=False,
 
 
 def get_server_list(args=None, skip_inactive=False, skip_disabled=False,
+                    skip_passive=False,
                     on_error_stop=True, suppress_error=False):
     """
     Get the server list from the configuration
@@ -940,6 +1122,7 @@ def get_server_list(args=None, skip_inactive=False, skip_disabled=False,
     :param args: an argparse namespace containing a list server_name parameter
     :param bool skip_inactive: skip inactive servers when 'all' is required
     :param bool skip_disabled: skip disabled servers when 'all' is required
+    :param bool skip_passive: skip passive servers when 'all' is required
     :param bool on_error_stop: stop if an error is found
     :param bool suppress_error: suppress display of errors (e.g. diagnose)
     :rtype: dict(str,barman.server.Server|None)
@@ -998,6 +1181,11 @@ def get_server_list(args=None, skip_inactive=False, skip_disabled=False,
                 output.info("Skipping temporarily disabled server '%s'"
                             % conf.name)
                 continue
+            # Skip passive nodes, if requested
+            if skip_passive and server_object.passive_node:
+                output.info("Skipping passive server '%s'",
+                            conf.name)
+                continue
             server_dict[server] = server_object
 
     return server_dict
@@ -1090,6 +1278,10 @@ def main():
                         '(defaults: %s)'
                         % ', '.join(barman.config.Config.CONFIG_FILES),
                    default=SUPPRESS)
+    p.add_argument('--color', '--colour',
+                   help='Whether to use colors in the output',
+                   choices=['never', 'always', 'auto'],
+                   default='auto')
     p.add_argument('-q', '--quiet', help='be quiet', action='store_true')
     p.add_argument('-d', '--debug', help='debug output', action='store_true')
     p.add_argument('-f', '--format', help='output format',
@@ -1100,6 +1292,7 @@ def main():
             archive_wal,
             backup,
             check,
+            check_backup,
             cron,
             delete,
             diagnose,
@@ -1107,6 +1300,7 @@ def main():
             list_backup,
             list_files,
             list_server,
+            put_wal,
             rebuild_xlogdb,
             receive_wal,
             recover,
@@ -1116,6 +1310,9 @@ def main():
             status,
             switch_wal,
             switch_xlog,
+            sync_info,
+            sync_backup,
+            sync_wals,
         ]
     )
     # noinspection PyBroadException
