@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2017 2ndQuadrant Limited
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -79,7 +79,7 @@ class StreamLineProcessor(object):
             # Handle the last line (always incomplete, maybe empty)
             self._handler(self._buf)
             return True
-        self._buf += data.decode('utf-8')
+        self._buf += data.decode('utf-8', 'replace')
         # If no '\n' is present, we just read a part of a very long line.
         # Nothing to do at the moment.
         if '\n' not in self._buf:
@@ -334,6 +334,7 @@ class Command(object):
                      check=False, *args, **kwargs)
         self.out = '\n'.join(out)
         self.err = '\n'.join(err)
+
         _logger.debug("Command stdout: %s", self.out)
         _logger.debug("Command stderr: %s", self.err)
 
@@ -886,6 +887,8 @@ class PgBaseBackup(PostgreSQLClient):
             # We don't need it because Barman already stores the full
             # WAL stream, so we disable this feature to avoid wasting one slot.
             self.args += ['--no-slot']
+            # We also need to specify that we do not want to fetch any WAL file
+            self.args += ['--wal-method=none']
 
         # The tablespace mapping option is repeated once for each tablespace
         if tbs_mapping:
