@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -56,6 +56,7 @@ _TIME_INTERVAL_RE = re.compile(r"""
       (\d+)\s+(day|month|week)s?  # N (day|month|week) with optional 's'
       \s*$
       """, re.IGNORECASE | re.VERBOSE)
+_SLOT_NAME_RE = re.compile("^[0-9a-z_]+$")
 
 REUSE_BACKUP_VALUES = ('copy', 'link', 'off')
 
@@ -142,28 +143,15 @@ class BackupOptions(CsvOption):
     # constants containing labels for allowed values
     EXCLUSIVE_BACKUP = 'exclusive_backup'
     CONCURRENT_BACKUP = 'concurrent_backup'
+    EXTERNAL_CONFIGURATION = 'external_configuration'
 
     # list holding all the allowed values for the BackupOption class
-    value_list = [EXCLUSIVE_BACKUP, CONCURRENT_BACKUP]
+    value_list = [EXCLUSIVE_BACKUP, CONCURRENT_BACKUP, EXTERNAL_CONFIGURATION]
     # map holding all the possible conflicts between the allowed values
     conflicts = {
         EXCLUSIVE_BACKUP: CONCURRENT_BACKUP,
         CONCURRENT_BACKUP: EXCLUSIVE_BACKUP, }
 
-    def validate(self, key, source):
-        """
-        Validates backup_option values: currently it makes sure
-        that either exclusive_backup or concurrent_backup are set.
-        """
-        if len(self) == 0:
-            return
-        if self.CONCURRENT_BACKUP not in self \
-                and self.EXCLUSIVE_BACKUP not in self:
-            raise ValueError("Invalid configuration value for "
-                             "key %s in %s: it must contain either "
-                             "exclusive_backup or concurrent_backup option"
-                             % (key, source))
-
 
 class RecoveryOptions(CsvOption):
     """
@@ -260,6 +248,25 @@ def parse_backup_method(value):
             "', '".join(BACKUP_METHOD_VALUES)))
 
 
+def parse_slot_name(value):
+    """
+    Replication slot names may only contain lower case letters, numbers,
+    and the underscore character. This function parse a replication slot name
+
+    :param str value: slot_name value
+    :return:
+    """
+    if value is None:
+        return None
+
+    value = value.lower()
+    if not _SLOT_NAME_RE.match(value):
+        raise ValueError(
+            "Replication slot names may only contain lower case letters, "
+            "numbers, and the underscore character.")
+    return value
+
+
 class ServerConfig(object):
     """
     This class represents the configuration for a specific Server instance.
@@ -287,17 +294,32 @@ class ServerConfig(object):
         'immediate_checkpoint',
         'incoming_wals_directory',
         'last_backup_maximum_age',
+        'max_incoming_wals_queue',
         'minimum_redundancy',
         'network_compression',
+        'parallel_jobs',
         'path_prefix',
         'post_archive_retry_script',
         'post_archive_script',
         'post_backup_retry_script',
         'post_backup_script',
+        'post_delete_script',
+        'post_delete_retry_script',
+        'post_recovery_retry_script',
+        'post_recovery_script',
+        'post_wal_delete_script',
+        'post_wal_delete_retry_script',
         'pre_archive_retry_script',
         'pre_archive_script',
         'pre_backup_retry_script',
         'pre_backup_script',
+        'pre_delete_script',
+        'pre_delete_retry_script',
+        'pre_recovery_retry_script',
+        'pre_recovery_script',
+        'pre_wal_delete_script',
+        'pre_wal_delete_retry_script',
+        'primary_ssh_command',
         'recovery_options',
         'retention_policy',
         'retention_policy_mode',
@@ -330,17 +352,32 @@ class ServerConfig(object):
         'custom_decompression_filter',
         'immediate_checkpoint',
         'last_backup_maximum_age',
+        'max_incoming_wals_queue',
         'minimum_redundancy',
         'network_compression',
+        'parallel_jobs',
         'path_prefix',
         'post_archive_retry_script',
         'post_archive_script',
         'post_backup_retry_script',
         'post_backup_script',
+        'post_delete_script',
+        'post_delete_retry_script',
+        'post_recovery_retry_script',
+        'post_recovery_script',
+        'post_wal_delete_script',
+        'post_wal_delete_retry_script',
         'pre_archive_retry_script',
         'pre_archive_script',
         'pre_backup_retry_script',
         'pre_backup_script',
+        'pre_delete_script',
+        'pre_delete_retry_script',
+        'pre_recovery_retry_script',
+        'pre_recovery_script',
+        'pre_wal_delete_script',
+        'pre_wal_delete_retry_script',
+        'primary_ssh_command',
         'recovery_options',
         'retention_policy',
         'retention_policy_mode',
@@ -371,6 +408,7 @@ class ServerConfig(object):
         'incoming_wals_directory': '%(backup_directory)s/incoming',
         'minimum_redundancy': '0',
         'network_compression': 'false',
+        'parallel_jobs': '1',
         'recovery_options': '',
         'retention_policy_mode': 'auto',
         'streaming_archiver': 'off',
@@ -399,11 +437,14 @@ class ServerConfig(object):
         'disabled': parse_boolean,
         'immediate_checkpoint': parse_boolean,
         'last_backup_maximum_age': parse_time_interval,
+        'max_incoming_wals_queue': int,
         'network_compression': parse_boolean,
+        'parallel_jobs': int,
         'recovery_options': RecoveryOptions,
         'reuse_backup': parse_reuse_backup,
         'streaming_archiver': parse_boolean,
         'streaming_archiver_batch_size': int,
+        'slot_name': parse_slot_name,
     }
 
     def invoke_parser(self, key, source, value, new_value):
@@ -509,10 +550,9 @@ class ServerConfig(object):
 
         if tablespace:
             # A tablespace can be copied using a per-tablespace bwlimit
-            tablespaces_bw_limit = self.tablespace_bandwidth_limit
-            if (tablespaces_bw_limit and
-                    tablespace.name in tablespaces_bw_limit):
-                bwlimit = tablespaces_bw_limit[tablespace.name]
+            tbl_bw_limit = self.tablespace_bandwidth_limit
+            if (tbl_bw_limit and tablespace.name in tbl_bw_limit):
+                bwlimit = tbl_bw_limit[tablespace.name]
 
         return bwlimit
 
