@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2017 2ndQuadrant Limited
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -182,6 +182,10 @@ class _RsyncCopyItem(object):
         self.analysis_start_time = None
         self.analysis_end_time = None
 
+        # Ensure that the user specified the item class, since it is mandatory
+        # to correctly handle the item
+        assert self.item_class
+
     def __str__(self):
         # Prepare strings for messages
         formatted_class = self.item_class
@@ -215,7 +219,7 @@ class RsyncCopyController(object):
     # This regular expression is used to parse each line of the output
     # of a "rsync --list-only" call. This regexp has been tested with any known
     # version of upstream rsync that is supported (>= 3.0.4)
-    LIST_ONLY_RE = re.compile("""
+    LIST_ONLY_RE = re.compile(r"""
         (?x) # Enable verbose mode
 
         ^ # start of the line
@@ -249,8 +253,9 @@ class RsyncCopyController(object):
     # vanished files that are not really an error. It is used because
     # in some cases rsync reports it with exit code 23 which could also mean
     # a fatal error
-    VANISHED_RE = re.compile("""
+    VANISHED_RE = re.compile(r"""
         (?x) # Enable verbose mode
+        (?i) # Case insensitive
 
         ^ # start of the line
         (
@@ -265,7 +270,7 @@ class RsyncCopyController(object):
         |
         # final summary
         rsync\ error:\ .* \(code\ 23\)\ at\ main\.c\(\d+\)
-            \ \[generator=[^\]]+\]
+            \ \[(generator|receiver)=[^\]]+\]
         )
         $ # end of the line
     """)
@@ -514,7 +519,7 @@ class RsyncCopyController(object):
             _logger.info("Copy interrupted by the user (safe before %s)",
                          self.safe_horizon)
             raise
-        except:
+        except BaseException:
             _logger.info("Copy failed (safe before %s)", self.safe_horizon)
             raise
         else:
@@ -729,7 +734,7 @@ class RsyncCopyController(object):
         """
         self.current_step += 1
         return "Copy step %s of %s: %s" % (
-                     self.current_step, self.total_steps, msg)
+            self.current_step, self.total_steps, msg)
 
     def _reuse_args(self, reuse_directory):
         """
@@ -866,8 +871,13 @@ class RsyncCopyController(object):
             # destination, rsync will discover the difference in any case.
             # It is then safe to skip checksum check here.
             dst_item = ref_hash.get(entry.path, None)
-            if (dst_item is None or dst_item.size != entry.size or
-                    dst_item.date != entry.date):
+            if dst_item is None:
+                item.safe_list.append(entry)
+                continue
+
+            different_size = dst_item.size != entry.size
+            different_date = dst_item.date != entry.date
+            if different_size or different_date:
                 item.safe_list.append(entry)
                 continue
 
@@ -1016,15 +1026,18 @@ class RsyncCopyController(object):
                 continue
             # Build a human readable name to refer to an item in the output
             ident = item.label
-            if (analysis_start is None or
-                    analysis_start > item.analysis_start_time):
+            if not analysis_start:
                 analysis_start = item.analysis_start_time
-            if (analysis_end is None or
-                    analysis_end < item.analysis_end_time):
+            elif analysis_start > item.analysis_start_time:
+                analysis_start = item.analysis_start_time
+
+            if not analysis_end:
                 analysis_end = item.analysis_end_time
+            elif analysis_end < item.analysis_end_time:
+                analysis_end = item.analysis_end_time
+
             stat['analysis_time_per_item'][ident] = total_seconds(
-                item.analysis_end_time -
-                item.analysis_start_time)
+                item.analysis_end_time - item.analysis_start_time)
         stat['analysis_time'] = total_seconds(analysis_end - analysis_start)
 
         # Calculate the time spent per job
