@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -21,12 +21,11 @@ This module represents the barman diagnostic tool.
 
 import json
 import logging
-import sys
 
 import barman
 from barman import fs, output
 from barman.backup import BackupInfo
-from barman.exceptions import FsOperationFailed
+from barman.exceptions import CommandFailedException, FsOperationFailed
 from barman.utils import BarmanEncoder
 
 _logger = logging.getLogger(__name__)
@@ -47,9 +46,12 @@ def exec_diagnose(servers, errors_list):
     # barman global config
     diagnosis['global']['config'] = dict(barman.__config__._global_config)
     diagnosis['global']['config']['errors_list'] = errors_list
-    command = fs.UnixLocalCommand()
-    # basic system info
-    diagnosis['global']['system_info'] = command.get_system_info()
+    try:
+        command = fs.UnixLocalCommand()
+        # basic system info
+        diagnosis['global']['system_info'] = command.get_system_info()
+    except CommandFailedException as e:
+        diagnosis['global']['system_info'] = {'error': repr(e)}
     diagnosis['global']['system_info']['barman_ver'] = barman.__version__
     # per server section
     for name in sorted(servers):
@@ -84,5 +86,5 @@ def exec_diagnose(servers, errors_list):
         }
         # Release any PostgreSQL resource
         server.close()
-    output.info(json.dumps(diagnosis, sys.stdout, cls=BarmanEncoder, indent=4,
+    output.info(json.dumps(diagnosis, cls=BarmanEncoder, indent=4,
                            sort_keys=True))
