@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2017 2ndQuadrant Limited
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -78,6 +78,12 @@ class LockFileException(BarmanException):
     """
 
 
+class SyncException(BarmanException):
+    """
+    Base Exception for synchronisation functions
+    """
+
+
 class DuplicateWalFile(WALFileException):
     """
     A duplicate WAL file has been found
@@ -109,6 +115,24 @@ class BackupInfoBadInitialisation(BackupException):
     """
 
 
+class SyncError(SyncException):
+    """
+    Synchronisation error
+    """
+
+
+class SyncNothingToDo(SyncException):
+    """
+    Nothing to do during sync operations
+    """
+
+
+class SyncToBeDeleted(SyncException):
+    """
+    An incomplete backup is to be deleted
+    """
+
+
 class CommandFailedException(CommandException):
     """
     Exception representing a failed command
@@ -150,11 +174,15 @@ class DataTransferFailure(CommandException):
         :param str msg: a descriptive message on what we are trying to do
         :return DataTransferFailure: will contain the message provided in msg
         """
-        details = msg
-        details += "\n%s error:\n" % cmd
-        details += e.args[0]['out']
-        details += e.args[0]['err']
-        return cls(details)
+        try:
+            details = msg
+            details += "\n%s error:\n" % cmd
+            details += e.args[0]['out']
+            details += e.args[0]['err']
+            return cls(details)
+        except (TypeError, NameError):
+            # If it is not a dictionary just convert it to a string
+            return cls(str(e.args))
 
 
 class CompressionIncompatibility(CompressionException):
@@ -299,3 +327,27 @@ class AbortedRetryHookScript(HookScriptException):
         return ("Abort '%s_%s' retry hook script (%s, exit code: %d)" % (
                 self.hook.phase, self.hook.name,
                 self.hook.script, self.hook.exit_status))
+
+
+class RecoveryException(BarmanException):
+    """
+    Exception for a recovery error
+    """
+
+
+class RecoveryTargetActionException(RecoveryException):
+    """
+    Exception for a wrong recovery target action
+    """
+
+
+class RecoveryStandbyModeException(RecoveryException):
+    """
+    Exception for a wrong recovery standby mode
+    """
+
+
+class RecoveryInvalidTargetException(RecoveryException):
+    """
+    Exception for a wrong recovery target
+    """
