@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2016 2ndQuadrant Italia Srl
+# Copyright (C) 2013-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -17,26 +17,12 @@
 
 import logging
 
-from barman.command_wrappers import Command
+from barman.command_wrappers import Command, full_command_quote
 from barman.exceptions import FsOperationFailed
 
 _logger = logging.getLogger(__name__)
 
 
-def _str(cmd_out):
-    """
-    Make a string from the output of a CommandWrapper execution.
-    If input is None returns a literal 'None' string
-
-    :param cmd_out: String or ByteString to convert
-    :return str: a string
-    """
-    if hasattr(cmd_out, 'decode') and callable(cmd_out.decode):
-        return cmd_out.decode('utf-8', 'replace')
-    else:
-        return str(cmd_out)
-
-
 class UnixLocalCommand(object):
     """
     This class is a wrapper for local calls for file system operations
@@ -44,7 +30,13 @@ class UnixLocalCommand(object):
 
     def __init__(self, path=None):
         # initialize a shell
-        self.cmd = Command(cmd='sh -c', shell=True, path=path)
+        self.internal_cmd = Command(cmd='sh', args=['-c'], path=path)
+
+    def cmd(self, cmd_name, args=[]):
+        """
+        Execute a command string, escaping it, if necessary
+        """
+        return self.internal_cmd(full_command_quote(cmd_name, args))
 
     def get_last_output(self):
         """
@@ -52,30 +44,28 @@ class UnixLocalCommand(object):
 
         :rtype: tuple[str,str]
         """
-        return _str(self.cmd.out), _str(self.cmd.err)
+        return self.internal_cmd.out, self.internal_cmd.err
 
     def create_dir_if_not_exists(self, dir_path):
         """
-            This method check for the existence of a directory.
-            if exist and is not a directory throws exception.
-            if is a directory everything is ok and no
-            mkdir operation is required.
-            Otherwise creates the directory using mkdir
-            if the mkdir fails an error is raised
+        This method recursively creates a directory if not exists
 
-            :param dir_path full path for the directory
+        If the path exists and is not a directory raise an exception.
+
+        :param str dir_path: full path for the directory
         """
         _logger.debug('Create directory %s if it does not exists' % dir_path)
         exists = self.exists(dir_path)
         if exists:
-            is_dir = self.cmd('test -d %s' % dir_path)
+            is_dir = self.cmd('test', args=['-d', dir_path])
             if is_dir != 0:
                 raise FsOperationFailed(
                     'A file with the same name already exists')
             else:
                 return False
         else:
-            mkdir_ret = self.cmd('mkdir %s' % dir_path)
+            # Make parent directories if needed
+            mkdir_ret = self.cmd('mkdir', args=['-p', dir_path])
             if mkdir_ret == 0:
                 return True
             else:
@@ -94,7 +84,7 @@ class UnixLocalCommand(object):
         _logger.debug('Delete path %s if exists' % path)
         exists = self.exists(path, False)
         if exists:
-            rm_ret = self.cmd('rm -fr %s' % path)
+            rm_ret = self.cmd('rm', args=['-fr', path])
             if rm_ret == 0:
                 return True
             else:
@@ -114,7 +104,7 @@ class UnixLocalCommand(object):
         _logger.debug('Check if directory %s exists' % dir_path)
         exists = self.exists(dir_path)
         if exists:
-            is_dir = self.cmd('test -d %s' % dir_path)
+            is_dir = self.cmd('test', args=['-d', dir_path])
             if is_dir != 0:
                 raise FsOperationFailed(
                     'A file with the same name exists, but is not a directory')
@@ -136,12 +126,13 @@ class UnixLocalCommand(object):
         _logger.debug('Check if directory %s is writable' % dir_path)
         exists = self.exists(dir_path)
         if exists:
-            is_dir = self.cmd('test -d %s' % dir_path)
+            is_dir = self.cmd('test', args=['-d', dir_path])
             if is_dir == 0:
-                can_write = self.cmd('touch %s/.barman_write_check' % dir_path)
+                can_write = self.cmd(
+                    'touch', args=["%s/.barman_write_check" % dir_path])
                 if can_write == 0:
                     can_remove = self.cmd(
-                        'rm %s/.barman_write_check' % dir_path)
+                        'rm', args=["%s/.barman_write_check" % dir_path])
                     if can_remove == 0:
                         return True
                     else:
@@ -165,12 +156,12 @@ class UnixLocalCommand(object):
             :param src full path to the source of the symlink
             :param dst full path for the destination of the symlink
         """
-        _logger.debug('Create symbolic link %s -> %s' % (src, dst))
+        _logger.debug('Create symbolic link %s -> %s' % (dst, src))
         exists = self.exists(src)
         if exists:
             exists_dst = self.exists(dst)
             if not exists_dst:
-                link = self.cmd('ln -s %s %s' % (src, dst))
+                link = self.cmd('ln', args=['-s', src, dst])
                 if link == 0:
                     return True
                 else:
@@ -185,32 +176,36 @@ class UnixLocalCommand(object):
             Gather important system information for 'barman diagnose' command
         """
         result = {}
-        # self.cmd.out can be None. The str() call will ensure it will be
-        # translated to a literal 'None'
+        # self.internal_cmd.out can be None. The str() call will ensure it
+        # will be translated to a literal 'None'
         release = ''
-        if self.cmd("lsb_release -a") == 0:
-            release = _str(self.cmd.out).rstrip()
+        if self.cmd("lsb_release", args=['-a']) == 0:
+            release = self.internal_cmd.out.rstrip()
         elif self.exists('/etc/lsb-release'):
-            self.cmd('cat /etc/lsb-release ')
-            release = "Ubuntu Linux %s" % _str(self.cmd.out).rstrip()
+            self.cmd('cat', args=['/etc/lsb-release'])
+            release = "Ubuntu Linux %s" % self.internal_cmd.out.rstrip()
         elif self.exists('/etc/debian_version'):
-            self.cmd('cat /etc/debian_version')
-            release = "Debian GNU/Linux %s" % _str(self.cmd.out).rstrip()
+            self.cmd('cat', args=['/etc/debian_version'])
+            release = "Debian GNU/Linux %s" % self.internal_cmd.out.rstrip()
         elif self.exists('/etc/redhat-release'):
-            self.cmd('cat /etc/redhat-release')
-            release = "RedHat Linux %s" % _str(self.cmd.out).rstrip()
+            self.cmd('cat', args=['/etc/redhat-release'])
+            release = "RedHat Linux %s" % self.internal_cmd.out.rstrip()
         elif self.cmd('sw_vers') == 0:
-            release = _str(self.cmd.out).rstrip()
+            release = self.internal_cmd.out.rstrip()
         result['release'] = release
 
-        self.cmd('uname -a')
-        result['kernel_ver'] = _str(self.cmd.out).rstrip()
-        self.cmd('python --version 2>&1')
-        result['python_ver'] = _str(self.cmd.out).rstrip()
-        self.cmd('rsync --version 2>&1')
-        result['rsync_ver'] = _str(self.cmd.out).splitlines(True)[0].rstrip()
-        self.cmd('ssh -V 2>&1')
-        result['ssh_ver'] = _str(self.cmd.out).rstrip()
+        self.cmd('uname', args=['-a'])
+        result['kernel_ver'] = self.internal_cmd.out.rstrip()
+        self.cmd('python', args=['--version', '2>&1'])
+        result['python_ver'] = self.internal_cmd.out.rstrip()
+        self.cmd('rsync', args=['--version', '2>&1'])
+        try:
+            result['rsync_ver'] = self.internal_cmd.out.splitlines(
+                True)[0].rstrip()
+        except IndexError:
+            result['rsync_ver'] = ''
+        self.cmd('ssh', args=['-V', '2>&1'])
+        result['ssh_ver'] = self.internal_cmd.out.rstrip()
         return result
 
     def get_file_content(self, path):
@@ -226,15 +221,15 @@ class UnixLocalCommand(object):
         if not result:
             raise FsOperationFailed('The %s file does not exist' % path)
 
-        result = self.cmd("test -r '%s'" % path)
+        result = self.cmd('test', args=['-r', path])
         if result != 0:
             raise FsOperationFailed('The %s file is not readable' % path)
 
-        result = self.cmd("cat '%s'" % path)
+        result = self.cmd('cat', args=[path])
         if result != 0:
             raise FsOperationFailed('Failed to execute "cat \'%s\'"' % path)
 
-        return self.cmd.out
+        return self.internal_cmd.out
 
     def exists(self, path, dereference=True):
         """
@@ -246,10 +241,10 @@ class UnixLocalCommand(object):
         :return bool: if the file exists or not.
         """
         _logger.debug('check for existence of: %s' % path)
-        cmd_str = "test -e '%s'" % path
+        options = ['-e', path]
         if not dereference:
-            cmd_str += " -o -L '%s'" % path
-        result = self.cmd(cmd_str)
+            options += ['-o', '-L', path]
+        result = self.cmd('test', args=options)
         return result == 0
 
     def ping(self):
@@ -263,18 +258,22 @@ class UnixLocalCommand(object):
         result = self.cmd("true")
         return result
 
-    def list_dir_content(self, dir_path, options=''):
+    def list_dir_content(self, dir_path, options=[]):
         """
         List the contents of a given directory.
 
         :param str dir_path: the path where we want the ls to be executed
-        :param str options: a string containing the options for the ls command
+        :param list[str] options: a string containing the options for the ls
+            command
         :return str: the ls cmd output
         """
         _logger.debug('list the content of a directory')
-        ls_command = "ls %s '%s'" % (options, dir_path)
-        self.cmd(ls_command)
-        return self.cmd.out
+        ls_options = []
+        if options:
+            ls_options += options
+        ls_options.append(dir_path)
+        self.cmd('ls', args=ls_options)
+        return self.internal_cmd.out
 
 
 class UnixRemoteCommand(UnixLocalCommand):
@@ -300,10 +299,10 @@ class UnixRemoteCommand(UnixLocalCommand):
 
         if ssh_command is None:
             raise FsOperationFailed('No ssh command provided')
-        self.cmd = Command(ssh_command,
-                           ssh_options,
-                           path=path,
-                           shell=True)
+        self.internal_cmd = Command(ssh_command,
+                                    args=ssh_options,
+                                    path=path,
+                                    shell=True)
         try:
             ret = self.cmd("true")
         except OSError:
