@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2017 2ndQuadrant Limited
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -19,6 +19,7 @@
 This module contains the logic to run hook scripts
 """
 
+import json
 import logging
 import time
 
@@ -89,10 +90,20 @@ class HookScriptRunner(object):
                 previous_backup_id = ''
         except UnknownBackupIdException:
             previous_backup_id = ''
+        try:
+            next_backup = self.backup_manager.get_next_backup(
+                backup_info.backup_id)
+            if next_backup:
+                next_backup_id = next_backup.backup_id
+            else:
+                next_backup_id = ''
+        except UnknownBackupIdException:
+            next_backup_id = ''
         self.environment.update({
             'BARMAN_BACKUP_DIR': backup_info.get_basebackup_directory(),
             'BARMAN_BACKUP_ID': backup_info.backup_id,
             'BARMAN_PREVIOUS_ID': previous_backup_id,
+            'BARMAN_NEXT_ID': next_backup_id,
             'BARMAN_STATUS': backup_info.status,
             'BARMAN_ERROR': backup_info.error or '',
         })
@@ -115,6 +126,41 @@ class HookScriptRunner(object):
             'BARMAN_ERROR': str(error or '')
         })
 
+    def env_from_recover(self, backup_info, dest, tablespaces, remote_command,
+                         error=None, **kwargs):
+        """
+        Prepare the environment for executing a script
+
+        :param BackupInfo backup_info: the backup metadata
+        :param str dest: the destination directory
+        :param dict[str,str]|None tablespaces: a tablespace name -> location
+            map (for relocation)
+        :param str|None remote_command: default None. The remote command
+            to recover the base backup, in case of remote backup.
+        :param str|Exception error: An error message in case of failure
+        """
+        self.env_from_backup_info(backup_info)
+
+        # Prepare a JSON representation of tablespace map
+        tablespaces_map = ''
+        if tablespaces:
+            tablespaces_map = json.dumps(tablespaces, sort_keys=True)
+
+        # Prepare a JSON representation of additional recovery options
+        # Skip any empty argument
+        kwargs_filtered = dict([(k, v) for k, v in kwargs.items() if v])
+        recover_options = ''
+        if kwargs_filtered:
+            recover_options = json.dumps(kwargs_filtered, sort_keys=True)
+
+        self.environment.update({
+            'BARMAN_DESTINATION_DIRECTORY': str(dest),
+            'BARMAN_TABLESPACES': tablespaces_map,
+            'BARMAN_REMOTE_COMMAND': str(remote_command or ''),
+            'BARMAN_RECOVER_OPTIONS': recover_options,
+            'BARMAN_ERROR': str(error or '')
+        })
+
     def run(self):
         """
         Run a a hook script if configured.
