@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2017 2ndQuadrant Limited
+# Copyright (C) 2013-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -17,6 +17,7 @@
 
 import ast
 import collections
+import inspect
 import logging
 import os
 
@@ -26,6 +27,7 @@ import dateutil.tz
 from barman import xlog
 from barman.compression import identify_compression
 from barman.exceptions import BackupInfoBadInitialisation
+from barman.utils import fsync_dir
 
 # Named tuple representing a Tablespace with 'name' 'oid' and 'location'
 # as property.
@@ -209,7 +211,7 @@ class FieldListFile(object):
         else:
             filename = filename or self.filename
             if filename:
-                info = open(filename, 'w')
+                info = open(filename + '.tmp', 'w')
             else:
                 info = None
 
@@ -218,14 +220,17 @@ class FieldListFile(object):
                 'either a valid filename or a file_object must be specified')
 
         with info:
-            for name in sorted(vars(type(self))):
-                field = getattr(type(self), name)
+            for name, field in sorted(inspect.getmembers(type(self))):
                 value = getattr(self, name, None)
                 if isinstance(field, Field):
                     if callable(field.to_str):
                         value = field.to_str(value)
                     info.write("%s=%s\n" % (name, value))
 
+        if not file_object:
+            os.rename(filename + '.tmp', filename)
+            fsync_dir(os.path.normpath(os.path.dirname(filename)))
+
     def load(self, filename=None, file_object=None):
         """
         Replaces the current object content with the one deserialized from
@@ -280,7 +285,7 @@ class FieldListFile(object):
                 if value == 'None':
                     value = None
                 elif isinstance(field, Field) and callable(field.from_str):
-                        value = field.from_str(value)
+                    value = field.from_str(value)
                 setattr(self, name, value)
 
     def items(self):
@@ -289,8 +294,7 @@ class FieldListFile(object):
 
         If a filed has a dump function defined, it will be used.
         """
-        for name in sorted(vars(type(self))):
-            field = getattr(type(self), name)
+        for name, field in sorted(inspect.getmembers(type(self))):
             value = getattr(self, name, None)
             if isinstance(field, Field):
                 if callable(field.to_str):
@@ -317,7 +321,7 @@ class WalFileInfo(FieldListFile):
     compression = Field('compression', doc='compression type')
 
     @classmethod
-    def from_file(cls, filename, default_compression=None, **kwargs):
+    def from_file(cls, filename, unidentified_compression=None, **kwargs):
         """
         Factory method to generate a WalFileInfo from a WAL file.
 
@@ -326,7 +330,7 @@ class WalFileInfo(FieldListFile):
         an AttributeError exception is raised.
 
         :param str filename: the file to inspect
-        :param str default_compression: the compression to set if
+        :param str unidentified_compression: the compression to set if
             the current schema is not identifiable.
         """
         stat = os.stat(filename)
@@ -335,7 +339,7 @@ class WalFileInfo(FieldListFile):
         kwargs.setdefault('time', stat.st_mtime)
         if 'compression' not in kwargs:
             kwargs['compression'] = identify_compression(filename) \
-                or default_compression
+                or unidentified_compression
         obj = cls(**kwargs)
         obj.filename = "%s.meta" % filename
         obj.orig_filename = filename
@@ -403,10 +407,13 @@ class BackupInfo(FieldListFile):
     EMPTY = 'EMPTY'
     STARTED = 'STARTED'
     FAILED = 'FAILED'
+    WAITING_FOR_WALS = 'WAITING_FOR_WALS'
     DONE = 'DONE'
-    STATUS_ALL = (EMPTY, STARTED, DONE, FAILED)
-    STATUS_NOT_EMPTY = (STARTED, DONE, FAILED)
-    STATUS_ARCHIVING = (STARTED, DONE)
+    SYNCING = 'SYNCING'
+    STATUS_COPY_DONE = (WAITING_FOR_WALS, DONE)
+    STATUS_ALL = (EMPTY, STARTED, WAITING_FOR_WALS, DONE, SYNCING, FAILED)
+    STATUS_NOT_EMPTY = (STARTED, WAITING_FOR_WALS, DONE, SYNCING, FAILED)
+    STATUS_ARCHIVING = (STARTED, WAITING_FOR_WALS, DONE, SYNCING)
 
     #: Status according to retention policies
     OBSOLETE = 'OBSOLETE'
@@ -557,11 +564,16 @@ class BackupInfo(FieldListFile):
         :param int tablespace_oid: the oid of a valid tablespace
         """
         # Check if a tablespace oid is passed and if is a valid oid
-        if tablespace_oid is not None and (
-                self.tablespaces is None or
-                all(str(tablespace_oid) != str(tablespace.oid)
-                    for tablespace in self.tablespaces)):
-            raise ValueError("Invalid tablespace OID %s" % tablespace_oid)
+        if tablespace_oid is not None:
+            if self.tablespaces is None:
+                raise ValueError("Invalid tablespace OID %s" % tablespace_oid)
+
+            invalid_oid = all(
+                str(tablespace_oid) != str(tablespace.oid)
+                for tablespace in self.tablespaces)
+            if invalid_oid:
+                raise ValueError("Invalid tablespace OID %s" % tablespace_oid)
+
         # Build the requested path according to backup_version value
         path = [self.get_basebackup_directory()]
         # Check te version of the backup
