@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2016 2ndQuadrant Italia Srl
+# Copyright (C) 2013-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -21,12 +21,13 @@ This module control how the output of Barman will be rendered
 
 from __future__ import print_function
 
+import datetime
 import inspect
 import logging
 import sys
 
 from barman.infofile import BackupInfo
-from barman.utils import pretty_size
+from barman.utils import human_readable_timedelta, pretty_size
 from barman.xlog import diff_lsn
 
 __all__ = [
@@ -42,6 +43,47 @@ error_occurred = False
 #: Exit code if error occurred
 error_exit_code = 1
 
+#: Enable colors in the output
+ansi_colors_enabled = False
+
+
+def _ansi_color(command):
+    """
+    Return the ansi sequence for the provided color
+    """
+    return '\033[%sm' % command
+
+
+def _colored(message, color):
+    """
+    Return a string formatted with the provided color.
+    """
+    if ansi_colors_enabled:
+        return _ansi_color(color) + message + _ansi_color('0')
+    else:
+        return message
+
+
+def _red(message):
+    """
+    Format a red string
+    """
+    return _colored(message, '31')
+
+
+def _green(message):
+    """
+    Format a green string
+    """
+    return _colored(message, '32')
+
+
+def _yellow(message):
+    """
+    Format a yellow string
+    """
+    return _colored(message, '33')
+
 
 def _format_message(message, args):
     """
@@ -371,19 +413,19 @@ class ConsoleOutputWriter(object):
         """
         Warning messages are sent to standard error
         """
-        self._err('WARNING: %s' % message, args)
+        self._err(_yellow('WARNING: %s' % message), args)
 
     def error(self, message, *args):
         """
         Error messages are sent to standard error
         """
-        self._err('ERROR: %s' % message, args)
+        self._err(_red('ERROR: %s' % message), args)
 
     def exception(self, message, *args):
         """
         Warning messages are sent to standard error
         """
-        self._err('EXCEPTION: %s' % message, args)
+        self._err(_red('EXCEPTION: %s' % message), args)
 
     def error_occurred(self):
         """
@@ -468,6 +510,12 @@ class ConsoleOutputWriter(object):
                       "'restore_command' can be executed by "
                       "the PostgreSQL user.")
         self.info("")
+        self.info(
+            "Recovery completed (start time: %s, elapsed time: %s)",
+            results['recovery_start_time'],
+            human_readable_timedelta(
+                datetime.datetime.now() - results['recovery_start_time']))
+        self.info("")
         self.info("Your PostgreSQL server has been successfully "
                   "prepared for recovery!")
 
@@ -511,11 +559,13 @@ class ConsoleOutputWriter(object):
         """
         self._record_check(server_name, check, status, hint)
         if hint:
-            self.info("\t%s: %s (%s)" %
-                      (check, 'OK' if status else 'FAILED', hint))
+            self.info(
+                "\t%s: %s (%s)" %
+                (check, _green('OK') if status else _red('FAILED'), hint))
         else:
-            self.info("\t%s: %s" %
-                      (check, 'OK' if status else 'FAILED'))
+            self.info(
+                "\t%s: %s" %
+                (check, _green('OK') if status else _red('FAILED')))
 
     def init_list_backup(self, server_name, minimal=False):
         """
@@ -545,7 +595,7 @@ class ConsoleOutputWriter(object):
 
         out_list = [
             "%s %s - " % (backup_info.server_name, backup_info.backup_id)]
-        if backup_info.status == BackupInfo.DONE:
+        if backup_info.status in BackupInfo.STATUS_COPY_DONE:
             end_time = backup_info.end_time.ctime()
             out_list.append('%s - Size: %s - WAL Size: %s' %
                             (end_time,
@@ -557,7 +607,9 @@ class ConsoleOutputWriter(object):
                                for tablespace in backup_info.tablespaces]
                 out_list.append(' (tablespaces: %s)' %
                                 ', '.join(tablespaces))
-            if retention_status:
+            if backup_info.status == BackupInfo.WAITING_FOR_WALS:
+                out_list.append(' - %s' % BackupInfo.WAITING_FOR_WALS)
+            if retention_status and retention_status != BackupInfo.NONE:
                 out_list.append(' - %s' % retention_status)
         else:
             out_list.append(backup_info.status)
@@ -577,7 +629,7 @@ class ConsoleOutputWriter(object):
         self.info("Backup %s:", data['backup_id'])
         self.info("  Server Name            : %s", data['server_name'])
         self.info("  Status                 : %s", data['status'])
-        if data['status'] == BackupInfo.DONE:
+        if data['status'] in BackupInfo.STATUS_COPY_DONE:
             self.info("  PostgreSQL Version     : %s", data['version'])
             self.info("  PGDATA directory       : %s", data['pgdata'])
             if data['tablespaces']:
@@ -592,8 +644,8 @@ class ConsoleOutputWriter(object):
                       pretty_size(data['size'] + data[
                           'wal_size']))
             if data['deduplicated_size'] is not None and data['size'] > 0:
-                deduplication_ratio = 1 - (float(data['deduplicated_size']) /
-                                           data['size'])
+                deduplication_ratio = (
+                    1 - (float(data['deduplicated_size']) / data['size']))
                 self.info("    Incremental size     : %s (-%s)",
                           pretty_size(data['deduplicated_size']),
                           '{percent:.2%}'.format(percent=deduplication_ratio)
@@ -611,13 +663,32 @@ class ConsoleOutputWriter(object):
             self.info("    Begin time           : %s",
                       data['begin_time'])
             self.info("    End time             : %s", data['end_time'])
+            # If copy statistics are available print a summary
+            copy_stats = data.get('copy_stats')
+            if copy_stats:
+                copy_time = copy_stats.get('copy_time')
+                if copy_time:
+                    value = human_readable_timedelta(
+                        datetime.timedelta(seconds=copy_time))
+                    # Show analysis time if it is more than a second
+                    analysis_time = copy_stats.get('analysis_time')
+                    if analysis_time is not None and analysis_time >= 1:
+                        value += " + %s startup" % (human_readable_timedelta(
+                            datetime.timedelta(seconds=analysis_time)))
+                    self.info("    Copy time            : %s", value)
+                    size = data['deduplicated_size'] or data['size']
+                    value = "%s/s" % pretty_size(size / copy_time)
+                    number_of_workers = copy_stats.get('number_of_workers', 1)
+                    if number_of_workers > 1:
+                        value += " (%s jobs)" % number_of_workers
+                    self.info("    Estimated throughput : %s", value)
             self.info("    Begin Offset         : %s",
                       data['begin_offset'])
             self.info("    End Offset           : %s",
                       data['end_offset'])
-            self.info("    Begin XLOG           : %s",
+            self.info("    Begin LSN           : %s",
                       data['begin_xlog'])
-            self.info("    End XLOG             : %s", data['end_xlog'])
+            self.info("    End LSN             : %s", data['end_xlog'])
             self.info("")
             self.info("  WAL information:")
             self.info("    No of files          : %s",
@@ -642,26 +713,29 @@ class ConsoleOutputWriter(object):
                     ", ".join([str(history.tli) for history in timelines]))
             self.info("")
             self.info("  Catalog information:")
-            self.info("    Retention Policy     : %s",
-                      data['retention_policy_status'] or
-                      'not enforced')
-            self.info("    Previous Backup      : %s",
-                      data.setdefault('previous_backup_id', 'not available') or
-                      '- (this is the oldest base backup)')
-            self.info("    Next Backup          : %s",
-                      data.setdefault('next_backup_id', 'not available') or
-                      '- (this is the latest base backup)')
+            self.info(
+                "    Retention Policy     : %s",
+                data['retention_policy_status'] or 'not enforced')
+            previous_backup_id = data.setdefault(
+                'previous_backup_id', 'not available')
+            self.info(
+                "    Previous Backup      : %s",
+                previous_backup_id or '- (this is the oldest base backup)')
+            next_backup_id = data.setdefault(
+                'next_backup_id', 'not available')
+            self.info(
+                "    Next Backup          : %s",
+                next_backup_id or '- (this is the latest base backup)')
+            if data['children_timelines']:
+                self.info("")
+                self.info(
+                    "WARNING: WAL information is inaccurate due to "
+                    "multiple timelines interacting with this backup")
         else:
             if data['error']:
                 self.info("  Error:            : %s",
                           data['error'])
 
-        if data['children_timelines']:
-            self.info("")
-            self.info(
-                "WARNING: WAL information is inaccurate due to "
-                "multiple timelines interacting with this backup")
-
     def init_status(self, server_name):
         """
         Init the status command
@@ -696,7 +770,7 @@ class ConsoleOutputWriter(object):
         """
         self.minimal = minimal
 
-    def result_replication_status(self, server_name, target, xlog_location,
+    def result_replication_status(self, server_name, target, server_lsn,
                                   standby_info):
         """
         Record a result line of a server status command
@@ -705,7 +779,7 @@ class ConsoleOutputWriter(object):
 
         :param str server_name: the replication server
         :param str target: all|hot-standby|wal-streamer
-        :param str xlog_location: server's xlog location
+        :param str server_lsn: server's current lsn
         :param StatReplication standby_info: status info of a standby
         """
 
@@ -718,10 +792,10 @@ class ConsoleOutputWriter(object):
 
         if self.minimal:
             # Minimal output
-            if xlog_location:
-                # xlog location from the master
-                self.info("%s for master '%s' (xlog @ %s):",
-                          title.capitalize(), server_name, xlog_location)
+            if server_lsn:
+                # current lsn from the master
+                self.info("%s for master '%s' (LSN @ %s):",
+                          title.capitalize(), server_name, server_lsn)
             else:
                 # We are connected to a standby
                 self.info("%s for slave '%s':",
@@ -730,10 +804,10 @@ class ConsoleOutputWriter(object):
             # Full output
             self.info("Status of %s for server '%s':",
                       title, server_name)
-            # xlog location from the master
-            if xlog_location:
-                self.info("  Current xlog location on master: %s",
-                          xlog_location)
+            # current lsn from the master
+            if server_lsn:
+                self.info("  Current LSN on master: %s",
+                          server_lsn)
 
         if standby_info is not None and not len(standby_info):
             self.info("  No %s attached", title)
@@ -743,14 +817,14 @@ class ConsoleOutputWriter(object):
         if self.minimal:
             n = 1
             for standby in standby_info:
-                if not standby.replay_location:
+                if not standby.replay_lsn:
                     # WAL streamer
                     self.info("  %s. W) %s@%s S:%s W:%s P:%s AN:%s",
                               n,
                               standby.usename,
                               standby.client_addr or 'socket',
-                              standby.sent_location,
-                              standby.write_location,
+                              standby.sent_lsn,
+                              standby.write_lsn,
                               standby.sync_priority,
                               standby.application_name)
                 else:
@@ -760,9 +834,9 @@ class ConsoleOutputWriter(object):
                               standby.sync_state[0].upper(),
                               standby.usename,
                               standby.client_addr or 'socket',
-                              standby.sent_location,
-                              standby.flush_location,
-                              standby.replay_location,
+                              standby.sent_lsn,
+                              standby.flush_lsn,
+                              standby.replay_lsn,
                               standby.sync_priority,
                               standby.application_name)
                 n += 1
@@ -774,18 +848,18 @@ class ConsoleOutputWriter(object):
                 self.info("")
 
                 # Calculate differences in bytes
-                sent_diff = diff_lsn(standby.sent_location,
-                                     standby.current_location)
-                write_diff = diff_lsn(standby.write_location,
-                                      standby.current_location)
-                flush_diff = diff_lsn(standby.flush_location,
-                                      standby.current_location)
-                replay_diff = diff_lsn(standby.replay_location,
-                                       standby.current_location)
+                sent_diff = diff_lsn(standby.sent_lsn,
+                                     standby.current_lsn)
+                write_diff = diff_lsn(standby.write_lsn,
+                                      standby.current_lsn)
+                flush_diff = diff_lsn(standby.flush_lsn,
+                                      standby.current_lsn)
+                replay_diff = diff_lsn(standby.replay_lsn,
+                                       standby.current_lsn)
 
                 # Determine the sync stage of the client
                 sync_stage = None
-                if not standby.replay_location:
+                if not standby.replay_lsn:
                     client_type = 'WAL streamer'
                     max_level = 3
                 else:
@@ -843,21 +917,21 @@ class ConsoleOutputWriter(object):
                 if getattr(standby, 'backend_xmin', None):
                     self.info("     Standby's xmin  : %s",
                               standby.backend_xmin or '-')
-                if getattr(standby, 'sent_location', None):
-                    self.info("     Sent location   : %s (diff: %s)",
-                              standby.sent_location,
+                if getattr(standby, 'sent_lsn', None):
+                    self.info("     Sent LSN   : %s (diff: %s)",
+                              standby.sent_lsn,
                               pretty_size(sent_diff))
-                if getattr(standby, 'write_location', None):
-                    self.info("     Write location  : %s (diff: %s)",
-                              standby.write_location,
+                if getattr(standby, 'write_lsn', None):
+                    self.info("     Write LSN  : %s (diff: %s)",
+                              standby.write_lsn,
                               pretty_size(write_diff))
-                if getattr(standby, 'flush_location', None):
-                    self.info("     Flush location  : %s (diff: %s)",
-                              standby.flush_location,
+                if getattr(standby, 'flush_lsn', None):
+                    self.info("     Flush LSN  : %s (diff: %s)",
+                              standby.flush_lsn,
                               pretty_size(flush_diff))
-                if getattr(standby, 'replay_location', None):
-                    self.info("     Replay location : %s (diff: %s)",
-                              standby.replay_location,
+                if getattr(standby, 'replay_lsn', None):
+                    self.info("     Replay LSN : %s (diff: %s)",
+                              standby.replay_lsn,
                               pretty_size(replay_diff))
                 n += 1
 
