@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -42,6 +42,7 @@ from barman.exceptions import (ConninfoException, PostgresAppNameError,
 from barman.infofile import Tablespace
 from barman.remote_status import RemoteStatusMixin
 from barman.utils import simplify_version, with_metaclass
+from barman.xlog import DEFAULT_XLOG_SEG_SIZE
 
 # This is necessary because the CONFIGURATION_LIMIT_EXCEEDED constant
 # has been added in psycopg2 2.5, but Barman supports version 2.4.2+ so
@@ -79,6 +80,8 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
     This abstract class represents a generic interface to a PostgreSQL server.
     """
 
+    CHECK_QUERY = 'SELECT 1'
+
     def __init__(self, config, conninfo):
         """
         Abstract base class constructor for PostgreSQL interface.
@@ -91,6 +94,7 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
         self.config = config
         self.conninfo = conninfo
         self._conn = None
+        self.allow_reconnect = True
         # Build a dictionary with connection info parameters
         # This is mainly used to speed up search in conninfo
         try:
@@ -144,27 +148,59 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
         """
         Generic function for Postgres connection (using psycopg2)
         """
-        if not self._conn:
+
+        if not self._check_connection():
             try:
                 self._conn = psycopg2.connect(self.conninfo)
             # If psycopg2 fails to connect to the host,
             # raise the appropriate exception
             except psycopg2.DatabaseError as e:
                 raise PostgresConnectionError(str(e).strip())
-            # Register the connection to the live connections list
+            # Register the connection to the list of live connections
             _live_connections.append(self)
         return self._conn
 
+    def _check_connection(self):
+        """
+        Return false if the connection is broken
+
+        :rtype: bool
+        """
+        # If the connection is not present return False
+        if not self._conn:
+            return False
+
+        # Check if the connection works by running 'SELECT 1'
+        cursor = None
+        try:
+            cursor = self._conn.cursor()
+            cursor.execute(self.CHECK_QUERY)
+        except psycopg2.DatabaseError:
+            # Connection is broken, so we need to reconnect
+            self.close()
+            # Raise an error if reconnect is not allowed
+            if not self.allow_reconnect:
+                raise PostgresConnectionError(
+                    "Connection lost, reconnection not allowed")
+            return False
+        finally:
+            if cursor:
+                cursor.close()
+
+        return True
+
     def close(self):
         """
         Close the connection to PostgreSQL
         """
         if self._conn:
-            if self._conn.status == STATUS_IN_TRANSACTION:
-                self._conn.rollback()
-            self._conn.close()
-            self._conn = None
+            # If the connection is still alive, rollback and close it
+            if not self._conn.closed:
+                if self._conn.status == STATUS_IN_TRANSACTION:
+                    self._conn.rollback()
+                self._conn.close()
             # Remove the connection from the live connections list
+            self._conn = None
             _live_connections.remove(self)
 
     def _cursor(self, *args, **kwargs):
@@ -224,6 +260,8 @@ class StreamingConnection(PostgreSQL):
     This class represents a streaming connection to a PostgreSQL server.
     """
 
+    CHECK_QUERY = 'IDENTIFY_SYSTEM'
+
     def __init__(self, config):
         """
         Streaming connection constructor
@@ -240,6 +278,9 @@ class StreamingConnection(PostgreSQL):
         # Make sure we connect using the 'replication' option which
         # triggers streaming replication protocol communication
         self.conn_parameters['replication'] = 'true'
+        # ensure that the datestyle is set to iso, working around an
+        # issue in some psycopg2 versions
+        self.conn_parameters['options'] = '-cdatestyle=iso'
         # Override 'dbname' parameter. This operation is required to mimic
         # the behaviour of pg_receivexlog and pg_basebackup
         self.conn_parameters['dbname'] = 'replication'
@@ -252,10 +293,12 @@ class StreamingConnection(PostgreSQL):
 
         :returns: the connection to the server
         """
-        if not self._conn:
-            # Build a connection and set autocommit
-            self._conn = super(StreamingConnection, self).connect()
-            self._conn.autocommit = True
+        if self._check_connection():
+            return self._conn
+
+        # Build a connection and set autocommit
+        self._conn = super(StreamingConnection, self).connect()
+        self._conn.autocommit = True
         return self._conn
 
     def fetch_remote_status(self):
@@ -379,12 +422,13 @@ class PostgreSQLConnection(PostgreSQL):
         """
         Connect to the PostgreSQL server. It reuses an existing connection.
         """
-        if self._conn:
+        if self._check_connection():
             return self._conn
 
         self._conn = super(PostgreSQLConnection, self).connect()
-        if (self._conn.server_version >= 90000 and
-                'application_name' not in self.conn_parameters):
+        server_version = self._conn.server_version
+        use_app_name = 'application_name' in self.conn_parameters
+        if server_version >= 90000 and not use_app_name:
             try:
                 cur = self._conn.cursor()
                 cur.execute('SET application_name TO barman')
@@ -482,9 +526,10 @@ class PostgreSQLConnection(PostgreSQL):
             if not self.is_in_recovery:
                 cur.execute(
                     "SELECT location, "
-                    "(pg_xlogfile_name_offset(location)).*, "
+                    "({pg_walfile_name_offset}(location)).*, "
                     "CURRENT_TIMESTAMP AS timestamp "
-                    "FROM pg_current_xlog_location() AS location")
+                    "FROM {pg_current_wal_lsn}() AS location"
+                    .format(**self.name_map))
                 return cur.fetchone()
             else:
                 cur.execute(
@@ -492,7 +537,8 @@ class PostgreSQLConnection(PostgreSQL):
                     "NULL AS file_name, "
                     "NULL AS file_offset, "
                     "CURRENT_TIMESTAMP AS timestamp "
-                    "FROM pg_last_xlog_replay_location() AS location")
+                    "FROM {pg_last_wal_replay_lsn}() AS location"
+                    .format(**self.name_map))
                 return cur.fetchone()
         except (PostgresConnectionError, psycopg2.Error) as e:
             _logger.debug("Error retrieving current xlog "
@@ -512,6 +558,52 @@ class PostgreSQLConnection(PostgreSQL):
             return current_xlog_info['file_name']
         return None
 
+    @property
+    def xlog_segment_size(self):
+        """
+        Retrieve the size of one WAL file.
+
+        In PostgreSQL 11, users will be able to change the WAL size
+        at runtime. Up to PostgreSQL 10, included, the WAL size can be changed
+        at compile time
+
+        :return: The wal size (In bytes)
+        """
+
+        # Prior to PostgreSQL 8.4, the wal segment size was not configurable,
+        # even in compilation
+        if self.server_version < 80400:
+            return DEFAULT_XLOG_SEG_SIZE
+
+        try:
+            cur = self._cursor(cursor_factory=DictCursor)
+            # We can't use the `get_setting` method here, because it
+            # use `SHOW`, returning an human readable value such as "16MB",
+            # while we prefer a raw value such as 16777216.
+            cur.execute("SELECT setting "
+                        "FROM pg_settings "
+                        "WHERE name='wal_segment_size'")
+            result = cur.fetchone()
+            wal_segment_size = int(result[0])
+
+            # Prior to PostgreSQL 11, the wal segment size is returned in
+            # blocks
+            if self.server_version < 110000:
+                cur.execute("SELECT setting "
+                            "FROM pg_settings "
+                            "WHERE name='wal_block_size'")
+                result = cur.fetchone()
+                wal_block_size = int(result[0])
+
+                wal_segment_size *= wal_block_size
+
+            return wal_segment_size
+        except ValueError as e:
+            _logger.error("Error retrieving current xlog "
+                          "segment size: %s",
+                          str(e).strip())
+            return None
+
     @property
     def current_xlog_location(self):
         """
@@ -601,6 +693,7 @@ class PostgreSQLConnection(PostgreSQL):
         pg_query_keys = [
             'server_txt_version',
             'is_superuser',
+            'is_in_recovery',
             'current_xlog',
             'pgespresso_installed',
             'replication_slot_support',
@@ -608,10 +701,9 @@ class PostgreSQLConnection(PostgreSQL):
             'synchronous_standby_names',
         ]
         # Initialise the result dictionary setting all the values to None
-        result = dict.fromkeys(pg_superuser_settings +
-                               pg_settings +
-                               pg_query_keys,
-                               None)
+        result = dict.fromkeys(
+            pg_superuser_settings + pg_settings + pg_query_keys,
+            None)
         try:
             # check for wal_level only if the version is >= 9.0
             if self.server_version >= 90000:
@@ -627,6 +719,7 @@ class PostgreSQLConnection(PostgreSQL):
                 result[name] = self.get_setting(name)
 
             result['is_superuser'] = self.is_superuser
+            result['is_in_recovery'] = self.is_in_recovery
             result['server_txt_version'] = self.server_txt_version
             result['pgespresso_installed'] = self.has_pgespresso
             result['current_xlog'] = self.current_xlog_file_name
@@ -792,16 +885,18 @@ class PostgreSQLConnection(PostgreSQL):
             if self.server_version < 80400:
                 cur.execute(
                     "SELECT location, "
-                    "(pg_xlogfile_name_offset(location)).*, "
+                    "({pg_walfile_name_offset}(location)).*, "
                     "now() AS timestamp "
-                    "FROM pg_start_backup(%s) AS location",
+                    "FROM pg_start_backup(%s) AS location"
+                    .format(**self.name_map),
                     (label,))
             else:
                 cur.execute(
                     "SELECT location, "
-                    "(pg_xlogfile_name_offset(location)).*, "
+                    "({pg_walfile_name_offset}(location)).*, "
                     "now() AS timestamp "
-                    "FROM pg_start_backup(%s,%s) AS location",
+                    "FROM pg_start_backup(%s,%s) AS location"
+                    .format(**self.name_map),
                     (label, self.config.immediate_checkpoint))
 
             start_row = cur.fetchone()
@@ -882,9 +977,11 @@ class PostgreSQLConnection(PostgreSQL):
             cur = conn.cursor(cursor_factory=DictCursor)
             cur.execute(
                 "SELECT location, "
-                "(pg_xlogfile_name_offset(location)).*, "
+                "({pg_walfile_name_offset}(location)).*, "
                 "now() AS timestamp "
-                "FROM pg_stop_backup() AS location")
+                "FROM pg_stop_backup() AS location"
+                .format(**self.name_map)
+            )
 
             return cur.fetchone()
         except (PostgresConnectionError, psycopg2.Error) as e:
@@ -1004,9 +1101,9 @@ class PostgreSQLConnection(PostgreSQL):
                 'pgespresso_abort_backup() on your PostgreSQL '
                 'server' % msg)
 
-    def switch_xlog(self):
+    def switch_wal(self):
         """
-        Execute a pg_switch_xlog()
+        Execute a pg_switch_wal()
 
         To be SURE of the switch of a xlog, we collect the xlogfile name
         before and after the switch.
@@ -1014,7 +1111,7 @@ class PostgreSQLConnection(PostgreSQL):
         file has changed, it returns an empty string otherwise.
 
         The method returns None if something went wrong during the execution
-        of the pg_switch_xlog command.
+        of the pg_switch_wal command.
 
         :rtype: str|None
         """
@@ -1030,14 +1127,17 @@ class PostgreSQLConnection(PostgreSQL):
 
             cur = conn.cursor()
             # Collect the xlog file name before the switch
-            cur.execute('SELECT pg_xlogfile_name('
-                        'pg_current_xlog_insert_location())')
+            cur.execute('SELECT {pg_walfile_name}('
+                        '{pg_current_wal_insert_lsn}())'
+                        .format(**self.name_map))
             pre_switch = cur.fetchone()[0]
             # Switch
-            cur.execute('SELECT pg_xlogfile_name(pg_switch_xlog())')
+            cur.execute('SELECT {pg_walfile_name}({pg_switch_wal}())'
+                        .format(**self.name_map))
             # Collect the xlog file name after the switch
-            cur.execute('SELECT pg_xlogfile_name('
-                        'pg_current_xlog_insert_location())')
+            cur.execute('SELECT {pg_walfile_name}('
+                        '{pg_current_wal_insert_lsn}())'
+                        .format(**self.name_map))
             post_switch = cur.fetchone()[0]
             if pre_switch < post_switch:
                 return pre_switch
@@ -1045,7 +1145,8 @@ class PostgreSQLConnection(PostgreSQL):
                 return ''
         except (PostgresConnectionError, psycopg2.Error) as e:
             _logger.debug(
-                "Error issuing pg_switch_xlog() command: %s",
+                "Error issuing {pg_switch_wal}() command: %s"
+                .format(**self.name_map),
                 str(e).strip())
             return None
 
@@ -1094,10 +1195,10 @@ class PostgreSQLConnection(PostgreSQL):
             # - backend_start
             # - backend_xmin (9.4+)
             # - state
-            # - sent_location
-            # - write_location
-            # - flush_location
-            # - replay_location
+            # - sent_lsn (sent_location before 10)
+            # - write_lsn (write_location before 10)
+            # - flush_lsn (flush_location before 10)
+            # - replay_lsn (replay_location before 10)
             # - sync_priority
             # - sync_state
             #
@@ -1106,76 +1207,115 @@ class PostgreSQLConnection(PostgreSQL):
                 raise PostgresUnsupportedFeature('9.1')
 
             from_repslot = ""
-            if self.server_version >= 90500:
-                # Current implementation (9.5+)
+            if self.server_version >= 100000:
+                # Current implementation (10+)
                 what = "r.*, rs.slot_name"
                 # Look for replication slot name
                 from_repslot = "LEFT JOIN pg_replication_slots rs " \
                                "ON (r.pid = rs.active_pid) "
+            elif self.server_version >= 90500:
+                # PostgreSQL 9.5/9.6
+                what = "pid, " \
+                    "usesysid, " \
+                    "usename, " \
+                    "application_name, " \
+                    "client_addr, " \
+                    "client_hostname, " \
+                    "client_port, " \
+                    "backend_start, " \
+                    "backend_xmin, " \
+                    "state, " \
+                    "sent_location AS sent_lsn, " \
+                    "write_location AS write_lsn, " \
+                    "flush_location AS flush_lsn, " \
+                    "replay_location AS replay_lsn, " \
+                    "sync_priority, " \
+                    "sync_state, " \
+                    "rs.slot_name"
+                # Look for replication slot name
+                from_repslot = "LEFT JOIN pg_replication_slots rs " \
+                               "ON (r.pid = rs.active_pid) "
             elif self.server_version >= 90400:
                 # PostgreSQL 9.4
-                what = "*"
+                what = "pid, " \
+                    "usesysid, " \
+                    "usename, " \
+                    "application_name, " \
+                    "client_addr, " \
+                    "client_hostname, " \
+                    "client_port, " \
+                    "backend_start, " \
+                    "backend_xmin, " \
+                    "state, " \
+                    "sent_location AS sent_lsn, " \
+                    "write_location AS write_lsn, " \
+                    "flush_location AS flush_lsn, " \
+                    "replay_location AS replay_lsn, " \
+                    "sync_priority, " \
+                    "sync_state"
             elif self.server_version >= 90200:
                 # PostgreSQL 9.2/9.3
-                what = "pid," \
-                    "usesysid," \
-                    "usename," \
-                    "application_name," \
-                    "client_addr," \
-                    "client_hostname," \
-                    "client_port," \
-                    "backend_start," \
-                    "CAST (NULL AS xid) AS backend_xmin," \
-                    "state," \
-                    "sent_location," \
-                    "write_location," \
-                    "flush_location," \
-                    "replay_location," \
-                    "sync_priority," \
-                    "sync_state "
+                what = "pid, " \
+                    "usesysid, " \
+                    "usename, " \
+                    "application_name, " \
+                    "client_addr, " \
+                    "client_hostname, " \
+                    "client_port, " \
+                    "backend_start, " \
+                    "CAST (NULL AS xid) AS backend_xmin, " \
+                    "state, " \
+                    "sent_location AS sent_lsn, " \
+                    "write_location AS write_lsn, " \
+                    "flush_location AS flush_lsn, " \
+                    "replay_location AS replay_lsn, " \
+                    "sync_priority, " \
+                    "sync_state"
             else:
                 # PostgreSQL 9.1
-                what = "procpid AS pid," \
-                    "usesysid," \
-                    "usename," \
-                    "application_name," \
-                    "client_addr," \
-                    "client_hostname," \
-                    "client_port," \
-                    "backend_start," \
-                    "CAST (NULL AS xid) AS backend_xmin," \
-                    "state," \
-                    "sent_location," \
-                    "write_location," \
-                    "flush_location," \
-                    "replay_location," \
-                    "sync_priority," \
-                    "sync_state "
+                what = "procpid AS pid, " \
+                    "usesysid, " \
+                    "usename, " \
+                    "application_name, " \
+                    "client_addr, " \
+                    "client_hostname, " \
+                    "client_port, " \
+                    "backend_start, " \
+                    "CAST (NULL AS xid) AS backend_xmin, " \
+                    "state, " \
+                    "sent_location AS sent_lsn, " \
+                    "write_location AS write_lsn, " \
+                    "flush_location AS flush_lsn, " \
+                    "replay_location AS replay_lsn, " \
+                    "sync_priority, " \
+                    "sync_state"
 
             # Streaming client
             if client_type == self.STANDBY:
                 # Standby server
-                where = 'WHERE replay_location IS NOT NULL '
+                where = 'WHERE {replay_lsn} IS NOT NULL '.format(
+                    **self.name_map)
             elif client_type == self.WALSTREAMER:
                 # WAL streamer
-                where = 'WHERE replay_location IS NULL '
+                where = 'WHERE {replay_lsn} IS NULL '.format(
+                    **self.name_map)
             else:
                 where = ''
 
             # Execute the query
             cur.execute(
                 "SELECT %s, "
-                "pg_is_in_recovery() AS is_in_recovery,"
+                "pg_is_in_recovery() AS is_in_recovery, "
                 "CASE WHEN pg_is_in_recovery() "
-                "  THEN NULL "
-                "  ELSE pg_current_xlog_location() "
-                "END AS current_location "
+                "  THEN {pg_last_wal_receive_lsn}() "
+                "  ELSE {pg_current_wal_lsn}() "
+                "END AS current_lsn "
                 "FROM pg_stat_replication r "
                 "%s"
                 "%s"
-                "ORDER BY sync_state DESC, sync_priority" % (what,
-                                                             from_repslot,
-                                                             where))
+                "ORDER BY sync_state DESC, sync_priority"
+                .format(**self.name_map)
+                % (what, from_repslot, where))
 
             # Generate a list of standby objects
             return cur.fetchall()
@@ -1232,6 +1372,9 @@ class PostgreSQLConnection(PostgreSQL):
         else:
             synchronous_standby_names = (
                 self.get_setting('synchronous_standby_names'))
+            # Return empty list if not defined
+            if synchronous_standby_names is None:
+                return []
             # Normalise the list of sync standby names
             # On PostgreSQL 9.6 it is possible to specify the number of
             # required synchronous standby using this format:
@@ -1245,3 +1388,53 @@ class PostgreSQLConnection(PostgreSQL):
                 names_end = len(synchronous_standby_names)
             names_list = synchronous_standby_names[names_start:names_end]
             return [x.strip() for x in names_list.split(',')]
+
+    @property
+    def name_map(self):
+        """
+        Return a map with function and directory names according to the current
+        PostgreSQL version.
+
+        Each entry has the `current` name as key and the name for the specific
+        version as value.
+
+        :rtype: dict[str]
+        """
+        # Avoid raising an error if the connection is not available
+        try:
+            server_version = self.server_version
+        except PostgresConnectionError:
+            _logger.debug('Impossible to detect the PostgreSQL version, '
+                          'name_map will return names from latest version')
+            server_version = None
+
+        if server_version and server_version < 100000:
+            return {
+                'pg_switch_wal': 'pg_switch_xlog',
+                'pg_walfile_name': 'pg_xlogfile_name',
+                'pg_wal': 'pg_xlog',
+                'pg_walfile_name_offset': 'pg_xlogfile_name_offset',
+                'pg_last_wal_replay_lsn': 'pg_last_xlog_replay_location',
+                'pg_current_wal_lsn': 'pg_current_xlog_location',
+                'pg_current_wal_insert_lsn': 'pg_current_xlog_insert_location',
+                'pg_last_wal_receive_lsn': 'pg_last_xlog_receive_location',
+                'sent_lsn': 'sent_location',
+                'write_lsn': 'write_location',
+                'flush_lsn': 'flush_location',
+                'replay_lsn': 'replay_location',
+            }
+
+        return {
+            'pg_switch_wal': 'pg_switch_wal',
+            'pg_walfile_name': 'pg_walfile_name',
+            'pg_wal': 'pg_wal',
+            'pg_walfile_name_offset': 'pg_walfile_name_offset',
+            'pg_last_wal_replay_lsn': 'pg_last_wal_replay_lsn',
+            'pg_current_wal_lsn': 'pg_current_wal_lsn',
+            'pg_current_wal_insert_lsn': 'pg_current_wal_insert_lsn',
+            'pg_last_wal_receive_lsn': 'pg_last_wal_receive_lsn',
+            'sent_lsn': 'sent_lsn',
+            'write_lsn': 'write_lsn',
+            'flush_lsn': 'flush_lsn',
+            'replay_lsn': 'replay_lsn',
+        }
