@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2017 2ndQuadrant Limited
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -22,6 +22,7 @@ This module contains the methods necessary to perform a recovery
 from __future__ import print_function
 
 import collections
+import datetime
 import logging
 import os
 import re
@@ -39,7 +40,10 @@ from barman.command_wrappers import RsyncPgData
 from barman.config import RecoveryOptions
 from barman.copy_controller import RsyncCopyController
 from barman.exceptions import (BadXlogSegmentName, CommandFailedException,
-                               DataTransferFailure, FsOperationFailed)
+                               DataTransferFailure, FsOperationFailed,
+                               RecoveryInvalidTargetException,
+                               RecoveryStandbyModeException,
+                               RecoveryTargetActionException)
 from barman.fs import UnixLocalCommand, UnixRemoteCommand
 from barman.infofile import BackupInfo
 from barman.utils import mkpath
@@ -83,17 +87,23 @@ class RecoveryExecutor(object):
         self.backup_manager = backup_manager
         self.server = backup_manager.server
         self.config = backup_manager.config
+        self.temp_dirs = []
 
-    def recover(self, backup_info, dest, tablespaces, target_tli,
-                target_time, target_xid, target_name,
-                target_immediate, exclusive, remote_command):
+    def recover(self, backup_info, dest, tablespaces=None, remote_command=None,
+                target_tli=None, target_time=None, target_xid=None,
+                target_name=None, target_immediate=False, exclusive=False,
+                target_action=None, standby_mode=None):
         """
         Performs a recovery of a backup
 
+        This method should be called in a closing context
+
         :param barman.infofile.BackupInfo backup_info: the backup to recover
         :param str dest: the destination directory
         :param dict[str,str]|None tablespaces: a tablespace
             name -> location map (for relocation)
+        :param str|None remote_command: The remote command to recover
+                               the base backup, in case of remote backup.
         :param str|None target_tli: the target timeline
         :param str|None target_time: the target time
         :param str|None target_xid: the target xid
@@ -102,8 +112,8 @@ class RecoveryExecutor(object):
         :param str|None target_immediate: end recovery as soon as consistency
             is reached
         :param bool exclusive: whether the recovery is exclusive or not
-        :param str|None remote_command: The remote command to recover
-                               the base backup, in case of remote backup.
+        :param str|None target_action: The recovery target action
+        :param bool|None standby_mode: standby mode
         """
 
         # Run the cron to be sure the wal catalog is up to date
@@ -114,6 +124,15 @@ class RecoveryExecutor(object):
                     backup_info.backup_id)
         output.info("Destination directory: %s", dest)
 
+        # If the backup we are recovering is still not validated and we
+        # haven't requested the get-wal feature, display a warning message
+        if not recovery_info['get_wal']:
+            if backup_info.status == BackupInfo.WAITING_FOR_WALS:
+                output.warning(
+                    "IMPORTANT: You have requested a recovery operation for "
+                    "a backup that does not have yet all the WAL files that "
+                    "are required for consistency.")
+
         # Set targets for PITR
         self._set_pitr_targets(recovery_info,
                                backup_info, dest,
@@ -121,7 +140,8 @@ class RecoveryExecutor(object):
                                target_time,
                                target_tli,
                                target_xid,
-                               target_immediate)
+                               target_immediate,
+                               target_action)
 
         # Retrieve the safe_horizon for smart copy
         self._retrieve_safe_horizon(recovery_info, backup_info, dest)
@@ -164,9 +184,27 @@ class RecoveryExecutor(object):
         else:
             backup_info.save(os.path.join(dest, '.barman-recover.info'))
 
+        # Standby mode is not available for PostgreSQL older than 9.0
+        if backup_info.version < 90000 and standby_mode:
+            raise RecoveryStandbyModeException(
+                'standby_mode is available only from PostgreSQL 9.0')
+
         # Restore the WAL segments. If GET_WAL option is set, skip this phase
         # as they will be retrieved using the wal-get command.
         if not recovery_info['get_wal']:
+            # If the backup we restored is still waiting for WALS, read the
+            # backup info again and check whether it has been validated.
+            # Notify the user if it is still not DONE.
+            if backup_info.status == BackupInfo.WAITING_FOR_WALS:
+                data = BackupInfo(self.server, backup_info.filename)
+                if data.status == BackupInfo.WAITING_FOR_WALS:
+                    output.warning(
+                        "IMPORTANT: The backup we have recovered IS NOT "
+                        "VALID. Required WAL files for consistency are "
+                        "missing. Please verify that WAL archiving is "
+                        "working correctly or evaluate using the 'get-wal' "
+                        "option for recovery")
+
             output.info("Copying required WAL segments.")
 
             try:
@@ -199,13 +237,16 @@ class RecoveryExecutor(object):
                                               remote_command,
                                               required_xlog_files)
 
-        # Generate recovery.conf file (only if needed by PITR)
-        if recovery_info['is_pitr']:
+        # Generate recovery.conf file (only if needed by PITR or get_wal)
+        is_pitr = recovery_info['is_pitr']
+        get_wal = recovery_info['get_wal']
+        if is_pitr or get_wal or standby_mode:
             output.info("Generating recovery.conf")
             self._generate_recovery_conf(recovery_info, backup_info, dest,
                                          target_immediate, exclusive,
                                          remote_command, target_name,
-                                         target_time, target_tli, target_xid)
+                                         target_time, target_tli, target_xid,
+                                         standby_mode)
 
         # Create archive_status directory if necessary
         archive_status_dir = os.path.join(recovery_info['wal_dest'],
@@ -235,9 +276,6 @@ class RecoveryExecutor(object):
                                           remote_command,
                                           recovery_info)
 
-        # Cleanup operations
-        self._teardown(recovery_info)
-
         return recovery_info
 
     def _setup(self, backup_info, remote_command, dest):
@@ -257,6 +295,9 @@ class RecoveryExecutor(object):
         else:
             wal_dest = os.path.join(dest, 'pg_wal')
 
+        tempdir = tempfile.mkdtemp(prefix='barman_recovery-')
+        self.temp_dirs.append(tempdir)
+
         recovery_info = {
             'cmd': None,
             'recovery_dest': 'local',
@@ -264,7 +305,7 @@ class RecoveryExecutor(object):
             'configuration_files': [],
             'destination_path': dest,
             'temporary_configuration_files': [],
-            'tempdir': tempfile.mkdtemp(prefix='barman_recovery-'),
+            'tempdir': tempdir,
             'is_pitr': False,
             'wal_dest': wal_dest,
             'get_wal': RecoveryOptions.GET_WAL in self.config.recovery_options,
@@ -277,6 +318,7 @@ class RecoveryExecutor(object):
             'delete_barman_xlog': False,
             'missing_files': [],
             'get_wal': False,
+            'recovery_start_time': datetime.datetime.now()
         }
         recovery_info['results'] = results
 
@@ -288,22 +330,17 @@ class RecoveryExecutor(object):
         # Handle remote recovery options
         if remote_command:
             recovery_info['recovery_dest'] = 'remote'
-            try:
-                recovery_info['rsync'] = RsyncPgData(
-                    path=self.server.path,
-                    ssh=remote_command,
-                    bwlimit=self.config.bandwidth_limit,
-                    network_compression=self.config.network_compression)
-            except CommandFailedException:
-                self._teardown(recovery_info)
-                raise
+            recovery_info['rsync'] = RsyncPgData(
+                path=self.server.path,
+                ssh=remote_command,
+                bwlimit=self.config.bandwidth_limit,
+                network_compression=self.config.network_compression)
 
             try:
                 # create a UnixRemoteCommand obj if is a remote recovery
                 recovery_info['cmd'] = UnixRemoteCommand(remote_command,
                                                          path=self.server.path)
             except FsOperationFailed:
-                self._teardown(recovery_info)
                 output.error(
                     "Unable to connect to the target host using the command "
                     "'%s'", remote_command)
@@ -316,7 +353,7 @@ class RecoveryExecutor(object):
 
     def _set_pitr_targets(self, recovery_info, backup_info, dest, target_name,
                           target_time, target_tli, target_xid,
-                          target_immediate):
+                          target_immediate, target_action):
         """
         Set PITR targets - as specified by the user
 
@@ -331,40 +368,46 @@ class RecoveryExecutor(object):
         :param str|None target_xid: recovery target transaction id for PITR
         :param bool|None target_immediate: end recovery as soon as consistency
             is reached
+        :param str|None target_action: recovery target action for PITR
         """
         target_epoch = None
         target_datetime = None
+        d_immediate = backup_info.version >= 90400 and target_immediate
+        d_tli = target_tli and target_tli != backup_info.timeline
         # Detect PITR
-        if (target_time or
-                target_xid or
-                (target_tli and target_tli != backup_info.timeline) or
-                target_name or
-                recovery_info['get_wal'] or
-                (backup_info.version >= 90400 and target_immediate)):
+        if target_time or target_xid or d_tli or target_name or d_immediate:
             recovery_info['is_pitr'] = True
             targets = {}
             if target_time:
-                # noinspection PyBroadException
                 try:
                     target_datetime = dateutil.parser.parse(target_time)
                 except ValueError as e:
-                    output.error(
-                        "unable to parse the target time parameter %r: %s",
-                        target_time, e)
-                    self._teardown(recovery_info)
-                    output.close_and_exit()
-                except Exception:
+                    raise RecoveryInvalidTargetException(
+                        "Unable to parse the target time parameter %r: %s" % (
+                            target_time, e))
+                except TypeError:
                     # this should not happen, but there is a known bug in
                     # dateutil.parser.parse() implementation
                     # ref: https://bugs.launchpad.net/dateutil/+bug/1247643
-                    output.error(
-                        "unable to parse the target time parameter %r",
+                    raise RecoveryInvalidTargetException(
+                        "Unable to parse the target time parameter %r" %
                         target_time)
-                    output.close_and_exit()
 
-                target_epoch = (
-                    time.mktime(target_datetime.timetuple()) +
-                    (target_datetime.microsecond / 1000000.))
+                # If the parsed timestamp is naive, forces it to local timezone
+                if target_datetime.tzinfo is None:
+                    target_datetime = target_datetime.replace(
+                        tzinfo=dateutil.tz.tzlocal())
+
+                # Check if the target time is reachable from the
+                # selected backup
+                if backup_info.end_time > target_datetime:
+                    raise RecoveryInvalidTargetException(
+                        "The requested target time %s "
+                        "is before the backup end time %s" %
+                        (target_datetime, backup_info.end_time))
+
+                ms = target_datetime.microsecond / 1000000.
+                target_epoch = time.mktime(target_datetime.timetuple()) + ms
                 targets['time'] = str(target_datetime)
             if target_xid:
                 targets['xid'] = str(target_xid)
@@ -374,6 +417,31 @@ class RecoveryExecutor(object):
                 targets['name'] = str(target_name)
             if target_immediate:
                 targets['immediate'] = target_immediate
+
+            # Manage the target_action option
+            if backup_info.version < 90100:
+                if target_action:
+                    raise RecoveryTargetActionException(
+                        "Illegal target action '%s' "
+                        "for this version of PostgreSQL" %
+                        target_action)
+            elif 90100 <= backup_info.version < 90500:
+                if target_action == 'pause':
+                    recovery_info['pause_at_recovery_target'] = "on"
+                elif target_action:
+                    raise RecoveryTargetActionException(
+                        "Illegal target action '%s' "
+                        "for this version of PostgreSQL" %
+                        target_action)
+            else:
+                if target_action in ('pause', 'shutdown', 'promote'):
+                    recovery_info['recovery_target_action'] = target_action
+                elif target_action:
+                    raise RecoveryTargetActionException(
+                        "Illegal target action '%s' "
+                        "for this version of PostgreSQL" %
+                        target_action)
+
             output.info(
                 "Doing PITR. Recovery target %s",
                 (", ".join(["%s: %r" % (k, v) for k, v in targets.items()])))
@@ -385,6 +453,12 @@ class RecoveryExecutor(object):
             if backup_info.version < 80400 and \
                     not recovery_info['get_wal']:
                 recovery_info['results']['delete_barman_xlog'] = True
+        else:
+            if target_action:
+                raise RecoveryTargetActionException(
+                    "Can't enable recovery target action when PITR "
+                    "is not required")
+
         recovery_info['target_epoch'] = target_epoch
         recovery_info['target_datetime'] = target_datetime
 
@@ -559,7 +633,7 @@ class RecoveryExecutor(object):
                     dst=dest_prefix + location,
                     bwlimit=self.config.get_bwlimit(tablespace),
                     item_class=controller.TABLESPACE_CLASS
-                    )
+                )
 
         # Add the PGDATA directory to the list of objects to be copied
         # by the controller
@@ -749,7 +823,7 @@ class RecoveryExecutor(object):
     def _generate_recovery_conf(self, recovery_info, backup_info, dest,
                                 immediate, exclusive, remote_command,
                                 target_name, target_time, target_tli,
-                                target_xid):
+                                target_xid, standby_mode):
         """
         Generate a recovery.conf file for PITR containing
         all the required configurations
@@ -767,6 +841,7 @@ class RecoveryExecutor(object):
         :param str target_time: recovery target time for PITR
         :param str target_tli: recovery target timeline for PITR
         :param str target_xid: recovery target transaction id for PITR
+        :param bool|None standby_mode: standby mode
         """
         if remote_command:
             recovery = open(os.path.join(recovery_info['tempdir'],
@@ -830,6 +905,20 @@ class RecoveryExecutor(object):
         if target_tli:
             print("recovery_target_timeline = %s" % target_tli, file=recovery)
 
+        # Write recovery target action
+        if 'pause_at_recovery_target' in recovery_info:
+            print("pause_at_recovery_target = '%s'" %
+                  recovery_info['pause_at_recovery_target'],
+                  file=recovery)
+        if 'recovery_target_action' in recovery_info:
+            print("recovery_target_action = '%s'" %
+                  recovery_info['recovery_target_action'],
+                  file=recovery)
+
+        # Writes the standby mode
+        if standby_mode:
+            print("standby_mode = 'on'", file=recovery)
+
         recovery.close()
         if remote_command:
             plain_rsync = RsyncPgData(
@@ -865,8 +954,9 @@ class RecoveryExecutor(object):
         # a configuration file is located outside the data dir.
         # This is not an error condition, so we check also for
         # `pg_ident.conf` which is an optional file.
-        for conf_file in (recovery_info['configuration_files'] +
-                          ['pg_hba.conf', 'pg_ident.conf']):
+        hardcoded_files = ['pg_hba.conf', 'pg_ident.conf']
+        conf_files = recovery_info['configuration_files'] + hardcoded_files
+        for conf_file in conf_files:
             source_path = os.path.join(
                 backup_info.get_data_directory(), conf_file)
             if not os.path.exists(source_path):
@@ -945,15 +1035,14 @@ class RecoveryExecutor(object):
                              e)
                 output.close_and_exit()
 
-    def _teardown(self, recovery_info):
+    def close(self):
         """
         Cleanup operations for a recovery
-
-        :param dict recovery_info: dictionary holding the basic values
-            for a recovery
         """
-        # Remove the temporary directory (created in the setup method)
-        shutil.rmtree(recovery_info['tempdir'])
+        # Remove the temporary directories
+        for temp_dir in self.temp_dirs:
+            shutil.rmtree(temp_dir, ignore_errors=True)
+        self.temp_dirs = []
 
     def _pg_config_mangle(self, filename, settings, backup_filename=None):
         """
