@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2017 2ndQuadrant Limited
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -23,6 +23,7 @@ import datetime
 import decimal
 import errno
 import grp
+import hashlib
 import json
 import logging
 import logging.handlers
@@ -214,9 +215,8 @@ def total_seconds(timedelta):
     if hasattr(timedelta, 'total_seconds'):
         return timedelta.total_seconds()
     else:
-        return (timedelta.microseconds +
-                (timedelta.seconds + timedelta.days * 24 * 3600) * 10**6
-                ) / 10.0**6
+        secs = (timedelta.seconds + timedelta.days * 24 * 3600) * 10**6
+        return (timedelta.microseconds + secs) / 10.0**6
 
 
 def which(executable, path=None):
@@ -228,9 +228,12 @@ def which(executable, path=None):
     :param str|None path: An optional search path to override the current one.
     :return str|None: the path of the executable or None
     """
-    # Get the system path and split.
+    # Get the system path if needed
     if path is None:
         path = os.getenv('PATH')
+    # If the path is None at this point we have nothing to search
+    if path is None:
+        return None
     # If executable is an absolute path, check if it exists and is executable
     # otherwise return failure.
     if os.path.isabs(executable):
@@ -238,7 +241,7 @@ def which(executable, path=None):
             return executable
         else:
             return None
-    # Search the requested executable in eery directory present in path and
+    # Search the requested executable in every directory present in path and
     # return the first occurrence that exists and is executable.
     for file_path in path.split(os.path.pathsep):
         file_path = os.path.join(file_path, executable)
@@ -303,6 +306,23 @@ def fsync_dir(dir_path):
     os.close(dir_fd)
 
 
+def fsync_file(file_path):
+    """
+    Execute fsync on a file ensuring it is synced to disk
+
+    Returns the file stats
+
+    :param str file_path: The file to sync
+    :return: file stat
+    :raise OSError: If something fails
+    """
+    file_fd = os.open(file_path, os.O_RDONLY)
+    file_stat = os.fstat(file_fd)
+    os.fsync(file_fd)
+    os.close(file_fd)
+    return file_stat
+
+
 def simplify_version(version_string):
     """
     Simplify a version number by removing the patch level
@@ -371,6 +391,10 @@ def is_power_of_two(number):
     Check if a number is a power of two or not
     """
 
+    # Returns None if number is set to None.
+    if number is None:
+        return None
+
     # This is a fast method to check for a power of two.
     #
     # A power of two has this structure:  100000 (one or more zeroes)
@@ -378,4 +402,22 @@ def is_power_of_two(number):
     # This is the bitwise and:            000000
     #
     # This is true only for every power of two
-    return number != 0 and (number & (number-1)) == 0
+    return number != 0 and (number & (number - 1)) == 0
+
+
+def file_md5(file_path, buffer_size=1024 * 16):
+    """
+    Calculate the md5 checksum for the provided file path
+
+    :param str file_path: path of the file to read
+    :param int buffer_size: read buffer size, default 16k
+    :return str: Hexadecimal md5 string
+    """
+    md5 = hashlib.md5()
+    with open(file_path, 'rb') as file_object:
+        while 1:
+            buf = file_object.read(buffer_size)
+            if not buf:
+                break
+            md5.update(buf)
+    return md5.hexdigest()
