@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2017 2ndQuadrant Limited
+# Copyright (C) 2011-2018 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -34,7 +34,7 @@ from barman.exceptions import (AbortedRetryHookScript, ArchiverFailure,
 from barman.hooks import HookScriptRunner, RetryHookScriptRunner
 from barman.infofile import WalFileInfo
 from barman.remote_status import RemoteStatusMixin
-from barman.utils import fsync_dir, mkpath, with_metaclass
+from barman.utils import fsync_dir, fsync_file, mkpath, with_metaclass
 
 _logger = logging.getLogger(__name__)
 
@@ -132,7 +132,8 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
 
         :param boolean verbose: Flag for verbose output
         """
-        compressor = self.backup_manager.compression_manager.get_compressor()
+        compressor = self.backup_manager.compression_manager \
+            .get_default_compressor()
         stamp = datetime.datetime.utcnow().strftime('%Y%m%dT%H%M%SZ')
         processed = 0
         header = "Processing xlog segments from %s for %s" % (
@@ -265,6 +266,8 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
         tmp_file = dst_file + '.tmp'
         dst_dir = os.path.dirname(dst_file)
 
+        comp_manager = self.backup_manager.compression_manager
+
         error = None
         try:
             # Run the pre_archive_script if present.
@@ -284,19 +287,18 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
             if os.path.exists(dst_file):
                 src_uncompressed = src_file
                 dst_uncompressed = dst_file
-                dst_info = WalFileInfo.from_file(dst_file)
+                dst_info = comp_manager.get_wal_file_info(dst_file)
                 try:
-                    comp_manager = self.backup_manager.compression_manager
                     if dst_info.compression is not None:
                         dst_uncompressed = dst_file + '.uncompressed'
-                        comp_manager.get_compressor(
-                            compression=dst_info.compression).decompress(
-                                dst_file, dst_uncompressed)
+                        comp_manager \
+                            .get_compressor(dst_info.compression) \
+                            .decompress(dst_file, dst_uncompressed)
                     if wal_info.compression:
                         src_uncompressed = src_file + '.uncompressed'
-                        comp_manager.get_compressor(
-                            compression=wal_info.compression).decompress(
-                                src_file, src_uncompressed)
+                        comp_manager \
+                            .get_compressor(wal_info.compression) \
+                            .decompress(src_file, src_uncompressed)
                     # Directly compare files.
                     # When the files are identical
                     # raise a MatchingDuplicateWalFile exception,
@@ -343,9 +345,7 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
                 wal_info.orig_filename = None
 
                 # Execute fsync() on the archived WAL file
-                file_fd = os.open(dst_file, os.O_RDONLY)
-                os.fsync(file_fd)
-                os.close(file_fd)
+                fsync_file(dst_file)
                 # Execute fsync() on the archived WAL containing directory
                 fsync_dir(dst_dir)
                 # Execute fsync() also on the incoming directory
@@ -482,6 +482,9 @@ class FileWalArchiver(WalArchiver):
         # Get the batch size from configuration (0 = unlimited)
         batch_size = self.config.archiver_batch_size
         # List and sort all files in the incoming directory
+        # IMPORTANT: the list is sorted, and this allows us to know that the
+        # WAL stream we have is monotonically increasing. That allows us to
+        # verify that a backup has all the WALs required for the restore.
         file_names = glob(os.path.join(
             self.config.incoming_wals_directory, '*'))
         file_names.sort()
@@ -550,22 +553,24 @@ class FileWalArchiver(WalArchiver):
         """
         Set additional status info - invoked by Server.status()
         """
-        remote_status = self.get_remote_status()
+        # We need to get full info here from the server
+        remote_status = self.server.get_remote_status()
 
         # If archive_mode is None, there are issues connecting to PostgreSQL
         if remote_status['archive_mode'] is None:
             return
 
-        output.result('status', self.config.name,
-                      "archive_command",
-                      "PostgreSQL 'archive_command' setting",
-                      remote_status['archive_command'] or
-                      "FAILED (please set it accordingly to documentation)")
+        output.result(
+            'status', self.config.name,
+            "archive_command",
+            "PostgreSQL 'archive_command' setting",
+            remote_status['archive_command'] or "FAILED "
+            "(please set it accordingly to documentation)")
         last_wal = remote_status.get('last_archived_wal')
         # If PostgreSQL is >= 9.4 we have the last_archived_time
         if last_wal and remote_status.get('last_archived_time'):
-                last_wal += ", at %s" % (
-                    remote_status['last_archived_time'].ctime())
+            last_wal += ", at %s" % (
+                remote_status['last_archived_time'].ctime())
         output.result('status', self.config.name,
                       "last_archived_wal",
                       "Last archived WAL",
@@ -792,7 +797,10 @@ class StreamingWalArchiver(WalArchiver):
         """
         # Get the batch size from configuration (0 = unlimited)
         batch_size = self.config.streaming_archiver_batch_size
-        # List and sort all files in the incoming directory
+        # List and sort all files in the incoming directory.
+        # IMPORTANT: the list is sorted, and this allows us to know that the
+        # WAL stream we have is monotonically increasing. That allows us to
+        # verify that a backup has all the WALs required for the restore.
         file_names = glob(os.path.join(
             self.config.streaming_wals_directory, '*'))
         file_names.sort()
@@ -807,6 +815,10 @@ class StreamingWalArchiver(WalArchiver):
             # Ignore temporary files
             if file_name.endswith('.tmp'):
                 continue
+            # If the file doesn't exist, it has been renamed/removed while
+            # we were reading the directory. Ignore it.
+            if not os.path.exists(file_name):
+                continue
             if not os.path.isfile(file_name):
                 errors.append(file_name)
             elif xlog.is_partial_file(file_name):
@@ -905,11 +917,11 @@ class StreamingWalArchiver(WalArchiver):
         # The receive-wal process is eligible for synchronous replication
         # if `synchronous_standby_names` is configured and contains
         # the value of `streaming_archiver_name`
-        synchronous = (syncnames and
-                       self.config.streaming_archiver_name in syncnames)
-        _logger.info('Synchronous WAL streaming for %s: %s',
-                     self.config.streaming_archiver_name,
-                     synchronous)
+        streaming_archiver_name = self.config.streaming_archiver_name
+        synchronous = (syncnames and streaming_archiver_name in syncnames)
+        _logger.debug('Synchronous WAL streaming for %s: %s',
+                      streaming_archiver_name,
+                      synchronous)
         return synchronous
 
     def status(self):
